unit DynVC;

interface

procedure ReadSMOCsParameters;
procedure RealTime_SMOCs;

implementation
uses Init_DC,DCUnit1, Acquisition, AcqThread,RealTime,Measurement,MathMyO;

//var
   { Parameters:}
//   SMOC_tau_raise,SMOC_tau_decay,SMOC_g_max,SMOC_V_rev,SMOC_rate,
//   al,be,
   { Variables: }
//   g_SMOC,m_SMOC,U_SMOC,nu_CaPulseTrain
//                                                                  :double;
//   OrderOfKinetics,IfPoisson,
//   i2                                                             :integer;

procedure ReadSMOCsParameters;
begin
  N_steps:=1;
  { Initial conditions }
  SMOC.g:=0;
  SMOC.m:=0;
  SMOC.U:=0;
  SMOC.nu_CaPulseTrain:=0;
  Randomize;
end;

procedure PulseTrain;
var i1 :integer;
    dt_next :double;
begin
  SMOC.nu_CaPulseTrain:=0;
  case SMOC.IfPoisson of
  0: if SMOC.rate>0 then begin // Deterministic
       i1     :=trunc((time+AcqTime*1e-6)/AcqTime);    //step number
       SMOC.i2:=trunc(1/SMOC.rate/AcqTime*1e3);        //number of steps between SMOCs
       if i1 mod SMOC.i2 = 0 then begin
          SMOC.nu_CaPulseTrain:=1/AcqTime*1e3;
       end;
     end;
  1: begin // Poisson Process
       i1:=trunc((time+AcqTime*1e-6)/AcqTime);    //step number
       if (i=0)or(i1=SMOC.i2) then begin
          if i<>0 then SMOC.nu_CaPulseTrain:=1/AcqTime*1e3;
          dt_next:=Random(200000)/100000/SMOC.rate*1e3; {ms}
          SMOC.i2:=i1+trunc(dt_next/AcqTime);          //step number of the next SMOC
          if SMOC.i2=i1 then SMOC.i2:=SMOC.i2+1;
       end;
     end;
  end;
end;

procedure RealTime_SMOCs;
var I_,G_,C_ :double;
begin
  { Stimulation }
  Define_I_G(time,I_,G_,C_);
  Icom:=I_ - G_*(V-Vus) - C_*(V-Vprev)/AcqTime;
  { SMOCs }
  PulseTrain;
  { Conductance integration }
  case SMOC.OrderOfKinetics of
  1: begin   // I-order kinetics
       if SMOC.nu_CaPulseTrain>0 then begin
          { Jump }
          SMOC.g:=SMOC.g + SMOC.g_max;
       end else begin
          { Decay }
          SMOC.g:=SMOC.g - AcqTime/SMOC.tau_decay*SMOC.g;
       end;
     end;
  2: begin   // II-order kinetics
       New_m2(0, SMOC.al,SMOC.be,AcqTime/1e3,SMOC.nu_CaPulseTrain, SMOC.m,SMOC.U);
       SMOC.g:=SMOC.g_max*SMOC.m;
     end;
  end;
  { *********************** }
  Icom:=Icom - SMOC.g*(V-SMOC.V_rev);   {pA}
end;

end.
