unit Init_DC;

interface

const ArrMax=10000000; nDrawMax=560000;
var
  IfNoPlotsInBackground
                                              :boolean;
  If_ThresholdClamp,If_FiringClamp,
  If_KillArtifacts,If_CondCompensation,
  If_NaAndK,If_Noise,IfAddIrealToIcom
                                              :integer;
  StartTime,EndTime,
  Protocol,DeviceNumber,DC_NOfFilesInTheList
                                              :integer;

  i_step,N_steps
                                              :integer;
  i,i_end
                                              :LongInt;
  Icom_scale,Ireal_scale,V_scale,
  V_offset,Icom_offset,Ireal_offset,
  Vmin,Vmax,dVmax,
  V,Ireal,Icom,Vprev,time,T_end,
  t_StartStim,t_EndStim,
  Vus,
  AcqTime,dt_draw,Epoch,
  uu,ss
                                              :double;
  Arr_V,Arr_Ireal,Arr_Icom,Arr_t,
  Arr_Vcorr,Arr_G1,Arr_Vmod                   :array[0..ArrMax] of double;
  DateTime                                    :TDateTime;
  Hour,Minute,Sec,MSec                        :Word;

{ PassiveProperties ========================================================== }
type
  PP_ParsVars=record
    { Parameters:}
    I_min,I_max,t1_start,t1_end,t2_start,t2_end,
    G_L,tau_m                                               :double;
    N_steps                                                 :integer;
    { Variables: }
    Vstep,Vbg,Icom_step                                     :double;
  end;

var
   PP                                                       :PP_ParsVars;

{ ThresholdClamp ============================================================= }
type
  TC_ParsVars=record
    { Parameters:}
    FixedThreshold,CurrentPulseAmplitude,
    CurrentPulseDuration,MinISI,
    { Variables: }
    t_AP                                                    :double;
  end;

var
   TC                                                       :TC_ParsVars;

{ SMOCs ====================================================================== }
type
  SMOC_Parameters=record
   { Parameters:}
   tau_raise,tau_decay,g_max,V_rev,rate,
   al,be
                                                           :double;
   OrderOfKinetics,IfPoisson                               :integer;
   { Variables: }
   g,m,U,nu_CaPulseTrain
                                                           :double;
   i2                                                      :integer;
  end;

var
  SMOC                                                     :SMOC_Parameters;

{ DynVC ====================================================================== }
type
  DynVC_Parameters=record
   { Parameters:}
   VL,Vh0,k,
   I_scale,Vh_scale,Vreal_scale,I_offset
                                                           :double;
   { Variables: }
   Vh,I,Vreal                                              :double;
  end;

var
  DynVC                                                     :DynVC_Parameters;

{ Na and K =================================================================== }
type
  NaK_ParsVars=record
    { Parameters:}
  g_GJ,Vshift,gNaP,gNa,gK,gAHP,
  gC
                                                       :double;
  TypeOfModNrn,TypeOfExpNrn                            :string;
  IfTurnOnModelNeuron,IfIncludeVrestInVshift,nchan     :integer;
  end;

var
   NaK                                                 :NaK_ParsVars;

{ FiringClamp ================================================================ }
type
  FC_ParsVars=record
   { Parameters:}
   dnt,dntUP,i_end
                                                        :integer;
   rate,Vmin,
   PositivePulseCurrent,NegativePulseCurrent,
   PositivePulseDuration                                :double;
   { Variables: }
   Meander
                                                        :double;
  end;

var
  FC                                                    :FC_ParsVars;

{ Artifacts ================================================================== }
type
  AF_ParsVars=record
   { Parameters:}
   alpha,dt_,tau_f,G2,
   { Variables: }
   KV3,KI2,KV,V_corr,G1
                                                       :double;
  end;

var
  AF                                                   :AF_ParsVars;

{ Electrode Compensation ===================================================== }
type
  EC_ParsVars=record
   { Parameters:}
   G1,tau_Ifilt,tau_Vfilt                              :double;
   Ireal_Or_Icom,NstepsOfDelayV                        :integer;
   { Variables: }
   Ireal_prev,Ireal_pprev,I_filt,V_filt                :double;
   V_delayed                                           :array[0..21] of double;
  end;

var
  EC                                                   :EC_ParsVars;

{ Noise ====================================================================== }
type
  Noise_ParsVars=record
   { Parameters:}
   tau_I,tau_G,sgm_I,sgm_G,
   { Variables: }
   I,G
                                                       :double;
  end;

var
  Noise                                                :Noise_ParsVars;

{ Current-Clamp ============================================================== }
type
  CC_ParsVars=record
    { Parameters:}
    I_stim,g_stim,C_stim,T_I,T_g
                                                       :double;
    TypeOfInputIndex                                   :integer;
  end;

var
   CC                                                  :CC_ParsVars;

{ Tonque ===================================================================== }
type
  US_ParsVars=record
    { Parameters:}
     u_min,u_max,s_min,s_max,
     t_StartStim_u,t_EndStim_u,
     t_StartStim_s,t_EndStim_s,
     T_pause
                                                       :double;
     Nu_steps,Ns_steps                                 :integer;
  end;

var
   US                                                  :US_ParsVars;

{ Series ===================================================================== }
type
  TResultsOfTrial=class(TObject)
    PP                                                 :PP_ParsVars;
    TC                                                 :TC_ParsVars;
    SMOC                                               :SMOC_Parameters;
    FC                                                 :FC_ParsVars;
    AF                                                 :AF_ParsVars;
    EC                                                 :EC_ParsVars;
    CC                                                 :CC_ParsVars;
    US                                                 :US_ParsVars;
  private
    { Private declarations }
  public
    { Public declarations }
    procedure RememberVariables(
    PP_                                                 :PP_ParsVars;
    TC_                                                 :TC_ParsVars;
    SMOC_                                               :SMOC_Parameters;
    FC_                                                 :FC_ParsVars;
    AF_                                                 :AF_ParsVars;
    EC_                                                 :EC_ParsVars;
    CC_                                                 :CC_ParsVars;
    US_                                                 :US_ParsVars);
  end;

var
  ResultsOfSeries            :array[0..200] of TResultsOfTrial;


implementation

  procedure TResultsOfTrial.RememberVariables(
    PP_                                                 :PP_ParsVars;
    TC_                                                 :TC_ParsVars;
    SMOC_                                               :SMOC_Parameters;
    FC_                                                 :FC_ParsVars;
    AF_                                                 :AF_ParsVars;
    EC_                                                 :EC_ParsVars;
    CC_                                                 :CC_ParsVars;
    US_                                                 :US_ParsVars);
  begin
    PP                                                 :=PP_;
    TC                                                 :=TC_;
    SMOC                                               :=SMOC_;
    FC                                                 :=FC_;
    AF                                                 :=AF_;
    EC                                                 :=EC_;
    CC                                                 :=CC_;
    US                                                 :=US_;
  end;

end.
