unit Measurement;

interface
uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs,StdCtrls, TeEngine, Series, ExtCtrls,
  TeeProcs, Chart,
  DDSpinEdit;

procedure ZeroArrays;
procedure ReadFiringClampParameters;
procedure ReadProtocolParameters;
procedure Measure_Init;
procedure Measure;
procedure AnalyseTrace;
procedure Define_I_G(time_:double; var I,G,C:double);
procedure ProvideMinimumRestingTime;

implementation
uses Init_DC,DCUnit1,DCUnit2, Acquisition, AcqThread,
     RealTime,CurrentClamp,PP_Analysis,Tongue,SMOCs,Na_and_K,
     ThresholdClamp,FiringClamp,Artifacts,MathMyO,OnlineComp,NoiseO,Dynamic_VC;

procedure ZeroArrays;
var i :integer;
begin
  for i:=0 to ArrMax do begin
      Arr_V[i]:=0; Arr_Ireal[i]:=0; Arr_Icom[i]:=0; Arr_t[i]:=0;
  end;
end;

procedure ReadFiringClampParameters;
begin
  FC.rate:=DCForm1.DDSpinEdit8.Value;
  FC.dnt:=trunc(1000/FC.rate/AcqTime);  { number of steps between APs }
  FC.PositivePulseDuration:=DCForm1.DDSpinEdit51.Value {ms};
  FC.dntUP:=trunc(FC.PositivePulseDuration/AcqTime);       { half-duration of "AP" }
  FC.PositivePulseCurrent:=DCForm1.DDSpinEdit12.Value*1000 {pA};
  FC.NegativePulseCurrent:=DCForm1.DDSpinEdit13.Value*1000 {pA};
  FC.Vmin:=DCForm1.DDSpinEdit11.Value {mV};
  FC.i_end:=i_end-trunc(50{ms}/AcqTime);
  FC.Meander:=0;
end;

procedure ReadProtocolParameters;
begin
  case DCForm1.RadioGroup2.ItemIndex of
  { BackGround }
  0: Read_BG_Parameters;
  { Current-Clamp }
  1: ReadCurrenClampParameters;
  { PassiveProperties }
  2: ReadPassiveParameters;
  { Tongue }
  3: ReadTongueParameters;
  { SMOCs }
  4: ReadSMOCsParameters;
  { Dynamic VC }
  5: Read_DynVC_Parameters;
  end;
  Protocol:=DCForm1.RadioGroup2.ItemIndex;
end;

procedure ReadSubProtocolParameters;
begin
  If_ThresholdClamp  :=IfTrue(DCForm1.CheckBox15.Checked);
  If_FiringClamp     :=IfTrue(DCForm1.CheckBox16.Checked);
  If_KillArtifacts   :=IfTrue(DCForm1.CheckBox17.Checked);
  If_CondCompensation:=IfTrue(DCForm1.CheckBox5.Checked);
  If_NaAndK          :=IfTrue(DCForm1.CheckBox6.Checked);
  If_Noise           :=IfTrue(DCForm1.CheckBox14.Checked);
  { Parameters }
  ReadThresholdClampParameters;
  ReadFiringClampParameters;
  ReadArtifactsParameters;
  ReadElectrodeCompensationParameters;
  Read_Na_and_K_Parameters;
  ReadNoiseParameters;
end;

procedure Measure_Init;
var
    Hour, Min, Sec, MSec: Word;
begin
  IfNoPlotsInBackground:=false;
  IfAddIrealToIcom:=IfTrue(DCForm1.CheckBox7.Checked);
  PP.I_min:=DCForm1.DDSpinEdit5.Value;
  PP.I_max:=DCForm1.DDSpinEdit6.Value;
  PP.N_steps:=trunc(DCForm1.DDSpinEdit7.Value);
  PP.t1_start:=DCForm1.DDSpinEdit14.Value;
  PP.t1_end  :=DCForm1.DDSpinEdit15.Value;
  PP.t2_start:=DCForm1.DDSpinEdit16.Value;
  PP.t2_end  :=DCForm1.DDSpinEdit17.Value;
  SMOC.tau_raise:=DCForm1.DDSpinEdit32.Value;  {ms}
  SMOC.tau_decay:=DCForm1.DDSpinEdit34.Value;  {ms}
  SMOC.g_max    :=DCForm1.DDSpinEdit35.Value;  {nS}
  SMOC.V_rev    :=DCForm1.DDSpinEdit36.Value;  {mV}
  SMOC.rate     :=DCForm1.DDSpinEdit33.Value;  {Hz}
  SMOC.OrderOfKinetics :=DCForm1.ComboBox3.ItemIndex+1;
  SMOC.IfPoisson       :=DCForm1.ComboBox4.ItemIndex;
  SMOC.al:=1/SMOC.tau_raise*1000;
  SMOC.be:=1/SMOC.tau_decay*1000;
  CC.TypeOfInputIndex:=DCForm1.ComboBox1.ItemIndex;
  CC.I_stim   :=DCForm1.DDSpinEdit1.Value;  {pA}
  CC.g_stim   :=DCForm1.DDSpinEdit37.Value; {nS}
  CC.C_stim   :=DCForm1.DDSpinEdit53.Value; {pF}
  CC.T_I      :=DCForm1.DDSpinEdit2.Value;  {ms}
  CC.T_g      :=DCForm1.DDSpinEdit52.Value;  {ms}
  Vus:=DCForm1.DDSpinEdit30.Value;{mV}
  PP.G_L:=DCForm1.DDSpinEdit18.Value;  {nS}
  PP.tau_m:=DCForm1.DDSpinEdit54.Value; {ms}
  US.u_min:=DCForm1.DDSpinEdit19.Value;
  US.u_max:=DCForm1.DDSpinEdit20.Value;
  US.s_min:=DCForm1.DDSpinEdit22.Value;
  US.s_max:=DCForm1.DDSpinEdit23.Value;
  US.Nu_steps:=trunc(DCForm1.DDSpinEdit21.Value);
  US.Ns_steps:=trunc(DCForm1.DDSpinEdit24.Value);
  T_end        :=DCForm1.DDSpinEdit3.Value;  {ms}
  AcqTime      :=DCForm1.DDSpinEdit4.Value;  {ms}
  t_StartStim  :=DCForm1.DDSpinEdit9.Value;  {ms}
  t_EndStim    :=DCForm1.DDSpinEdit10.Value;  {ms}
  US.t_StartStim_u:=DCForm1.DDSpinEdit59.Value;  {ms}
  US.t_EndStim_u  :=DCForm1.DDSpinEdit60.Value;  {ms}
  US.t_StartStim_s:=DCForm1.DDSpinEdit61.Value;  {ms}
  US.t_EndStim_s  :=DCForm1.DDSpinEdit62.Value;  {ms}
  US.T_pause   :=DCForm1.DDSpinEdit55.Value;  {ms}
  V_scale      :=DCForm1.DDSpinEdit25.Value;  {mV/V}
  Icom_scale   :=DCForm1.DDSpinEdit26.Value;  {pA/V}
  Ireal_scale  :=DCForm1.DDSpinEdit31.Value;  {pA/V}
  V_offset     :=DCForm1.DDSpinEdit39.Value;  {mV}
  Icom_offset  :=DCForm1.DDSpinEdit38.Value;  {pA}
  Ireal_offset :=DCForm1.DDSpinEdit40.Value;  {pA}
  Vmin         :=DCForm1.DDSpinEdit80.Value;  {mV}
  Vmax         :=DCForm1.DDSpinEdit81.Value;  {mV}
  dVmax        :=DCForm1.DDSpinEdit82.Value;  {mV}
  dt_draw      :=DCForm1.DDSpinEdit27.Value;  {ms}
  //if dt_draw=0 then dt_draw:=AcqTime;
  Epoch        :=DCForm1.DDSpinEdit28.Value*1e3;  {ms}
  DeviceNumber :=trunc(DCForm1.ComboBox2.ItemIndex);
  i_end:=trunc(T_end/AcqTime);
  {************************}
  ReadProtocolParameters;
  {************************}
  ReadSubProtocolParameters;
  {************************}
  Initialize_AI0_AI1_AO0(1000/AcqTime, DeviceNumber);
  //Icom:=0; V:=0;
  AI0_AI1_AO0(DeviceNumber);
  if IfInThread<>1 then DCForm1.InitializeDrawing;
  { Time }
  DecodeTime(Now, Hour, Min, Sec, MSec);
  StartTime:=Min*60000 + Sec*1000 + Msec;
end;

procedure Measure;
var
    Hour, Min, Sec, MSec: Word;
begin
  { Measure; }
  Measure_Init;
  i:=-1;
  repeat    i:=i+1;
    {******************************}
    RealTimeCalculations_at_OneStep;
    {******************************}
  until i>=i_end;
  { Print Time }
  DecodeTime(Now, Hour, Min, Sec, MSec);
  EndTime:=Min*60000 + Sec*1000 + Msec;
  DCForm1.Label5.Caption:='Control time='+IntToStr(EndTime-StartTime)+' ms';
  DCForm1.Label27.Caption:='Trace number '+IntToStr(i_step);
  { Stop Acquisition }
  Stop_AI0_AI1_AO0(DeviceNumber);
end;

procedure AnalyseTrace;
begin
  case Protocol of
  { BackGround }
  0: ;
  { Current-Clamp }
  1: ;
  { PassiveProperties }
  2: AnalysePassive;
  { Tongue }
  3: ;
  { SMOCs }
  4: ;
  { Dynamic VC }
  5: ;
  end;
end;

procedure Define_I_G(time_:double; var I,G,C:double);
begin
  case Protocol of
  { BackGround }
  0: ;
  { Current-Clamp }
  1,4: CC_Define_I_G(time_,I,G,C);
  { PassiveProperties }
  2: PP_Define_I_G(time_,I,G,C);
  { Tongue }
  3: US_Define_I_G(time_,I,G,C);
  { Dynamic VC }
  5: ;
  end;
end;

procedure ProvideMinimumRestingTime;
begin
  case Protocol of
  { BackGround }
  0: ;
  { Current-Clamp }
  1,4: ;
  { PassiveProperties }
  2: ;
  { Tongue }
  3: US_ProvideMinimumRestingTime;
  { Dynamic VC }
  5: ;
  end;
end;

end.
