unit Na_and_K;

interface
uses NeuronO,CondBasedO;

procedure Read_Na_and_K_Parameters;
procedure RealTime_Na_and_K;

const ArrMaxNaAndK=1000000;
var
   ANrn,  // Model neuron applied in the case of no DAQ-card.
   CNrn   // Model neuron used to drive recorded neuron through gap-junction
                         :TNeuron;
   BNrn                  :TCondBasedNrn;  // Additions to a real neuron
   Arr_NaAndK            :array[0..13,0..ArrMaxNaAndK] of double;
   i_draw_NaAndK         :integer;

implementation
uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, TeEngine, Series, ExtCtrls, TeeProcs, Chart,
  typeNrnParsO, t_dtO, ChannelO,Calmar_NaO,Calmar_KO,
//  HH_CalmarO,HH_DestO,HH_migliO,HH_ChowO,HH_LyleO,HH_NaundorfO,
//  HH_CumminsO,HH_WhiteO,HH_ShuO,
//  HH_PassiveO,
  CreateNrnO, SetNrnParsO,
  DDSpinEdit, Spin,
  DCUnit1, Acquisition, AcqThread,RealTime,Measurement,MathMyO,
  CurrentClamp,Init_DC;


var
   { Parameters:}
   NP                                 :NeuronProperties;
   { Variables: }
   eta,hiclose,hiopen                 :array[0..1000] of integer;
   I_Na_K_rest,INa,INaP,IAHP,IK,IC,
   EC,tau_open,tau_fast,dobk01_fast
                                                                  :double;


procedure InitCationCurrent;
var i :integer;
begin
  NaK.nchan:=trunc(DCForm1.DDSpinEdit66.Value);
  NaK.gC   :=DCForm1.DDSpinEdit65.Value;  {nS}
  EC:=0; {mV}
  tau_open:=0.2;   {ms}
  tau_fast:=1;     {ms}
  dobk01_fast:=1-exp(-dt*1e3/tau_fast);
  for i:=1 to NaK.nchan do eta[i]:=0;
  for i:=1 to 1000 do begin
      hiopen[i]:=0;
      hiclose[i]:=0;
  end;
  randomize;
end;

function CationCurrent(V_,t_:double):double;
var
    tau_slow,p,dobk01,dobk10
                      :double;
    sum_eta,l         :integer;
    close_fast        :boolean;

  function tau_slow_V(V_:double) :double;
  begin  tau_slow_V:=256/(exp((V_+90)/5)+exp(-(V_+90)/125))+4;  end;
begin
 sum_eta:=0;
 for l:=1 to NaK.nchan do begin
     p:=random;
     if eta[l]=0 then begin
        { fast closing }
        if (p<dobk01_fast) then begin
            eta[l]:=1;
        end;
        { slow closing }
        tau_slow:=tau_slow_V(V_);
        dobk01:=1-exp(-dt*1e3/tau_slow);
        if (p<dobk01) then begin
            eta[l]:=1;
        end;
     end else begin
        { opening }
        dobk10:=1-exp(-dt*1e3/tau_open*2);
        if p<dobk10 then begin
           eta[l]:=0;
        end;
     end;
     sum_eta:=sum_eta+eta[l];
 end;
 CationCurrent:=sum_eta*NaK.gC*(EC-V_);
end;

procedure Read_Na_and_K_Parameters;
begin
//  N_steps:=1;
  NaK.g_GJ  :=DCForm1.DDSpinEdit41.Value; {nS}
  NaK.Vshift:=DCForm1.DDSpinEdit45.Value {mV};
  NaK.gNa   :=DCForm1.DDSpinEdit43.Value; {nS}
  NaK.gNaP  :=DCForm1.DDSpinEdit63.Value; {nS}
  NaK.gAHP  :=DCForm1.DDSpinEdit72.Value; {nS}
  NaK.gK    :=DCForm1.DDSpinEdit44.Value; {nS}
  NaK.TypeOfExpNrn:=DCForm1.ComboBox6.Items[DCForm1.ComboBox6.ItemIndex]; //'Chow'
  NaK.TypeOfModNrn:=DCForm1.ComboBox5.Items[DCForm1.ComboBox5.ItemIndex]; //'Chow'
  NaK.IfTurnOnModelNeuron   :=IfTrue(DCForm1.CheckBox13.Checked);
  NaK.IfIncludeVrestInVshift:=IfTrue(DCForm1.CheckBox11.Checked);
  {**********}
  dt:=AcqTime/1e3;
  { Define the type of neurons A and C }
  NP.HH_type:=NaK.TypeOfModNrn;
  NP.HH_order:='1-point';
  { Set parameters according to the type of neuron and create neurons-objects }
  HodgkinPhysParameters(NP);
  CreateNeuronByTypeO(NP, ANrn);
  CreateNeuronByTypeO(NP, CNrn);
  ANrn.NP.Square:=DCForm1.DDSpinEdit42.Value/1e5; {cm^2}
  CNrn.NP.Square:=DCForm1.DDSpinEdit42.Value/1e5; {cm^2}
  { Create BNrn }
  NP.HH_type:=NaK.TypeOfExpNrn;
  NP.HH_order:='1-point';
  HodgkinPhysParameters(NP);
  BNrn:=TCondBasedNrn.Create(NP);
{    if      NP.HH_type='Calmar'   then BNrn:=TCalmarNrn.Create(NP)
    else if NP.HH_type='Destexhe' then BNrn:=TDestNrn.Create(NP)
    else if NP.HH_type='Lyle'     then BNrn:=TLyleNrn.Create(NP)
    else if NP.HH_type='Migliore' then BNrn:=TMigliNrn.Create(NP)
    else if NP.HH_type='Cummins'  then BNrn:=TCumNrn.Create(NP)
    else if NP.HH_type='Chow'     then BNrn:=TChowNrn.Create(NP)
    else if NP.HH_type='Naundorf' then BNrn:=TNaundorfNrn.Create(NP)
    else if NP.HH_type='White'    then BNrn:=TWhiteNrn.Create(NP)
    else if NP.HH_type='Shu'      then BNrn:=TShuNrn.Create(NP)
    else if NP.HH_type='Passive'  then BNrn:=TPassiveNrn.Create(NP)
    else BNrn:=nil;}
  BNrn.NP:=NP;
  { Initial conditions }
  ANrn.InitialConditions;
  CNrn.InitialConditions;
  BNrn.NP.gNa :=NaK.gNa;
  BNrn.NP.gNaP:=NaK.gNaP;
  BNrn.NP.gAHP:=NaK.gAHP;
  BNrn.NP.gK  :=NaK.gK;
  BNrn.NP.gL  :=PP.G_L; {nS}
  BNrn.InitialConditions;
  InitCationCurrent;
  { Shift voltage: Vshift=Vrest+dVshift }
  if (NaK.IfIncludeVrestInVshift=1)and(DeviceNumber<>0) then begin
      NaK.Vshift:=NaK.Vshift + BNrn.NV.V*1e3; {mV}
  end;
  INa :=-BNrn.chNa.Current;
  INaP:=-BNrn.chNaP.Current;
  IAHP:=-BNrn.chAHP.Current;
  IK  :=-BNrn.chK.Current;
  I_Na_K_rest:=INa + IK + INaP + IAHP;
  i_draw_NaAndK:=0;
end;

procedure RealTime_Na_and_K;
var
    Iind,Istim_,IGJ      :double;
begin
  { ================= REAL NEURON ================== }
  IGJ:=0;  INa:=0;  INaP:=0;  IAHP:=0;  IK:=0;  IC:=0;
  { Set current step }
  Istim_:=StimCurrent_CC(time,V,Vprev);
  { *** Add GJ-current *** }
  IGJ := - NaK.g_GJ*(V + NaK.Vshift - CNrn.NV.V*1e3);   {pA}
  { *** Add active currents *** }
  if not((BNrn.NP.gNa=0)and(BNrn.NP.gK=0)and
         (BNrn.NP.gNaP=0)and(BNrn.NP.gAHP=0)) then begin
     BNrn.NV.V:=(V+NaK.Vshift)/1e3;
     if (BNrn.NV.V>-0.1{V})and(BNrn.NV.V<0.05{V}) then begin
         INa :=-BNrn.chNa.Current;
         INaP:=-BNrn.chNaP.Current;
         IAHP:=-BNrn.chAHP.Current;
         IK  :=-BNrn.chK.Current;
         IC  :=CationCurrent(BNrn.NV.V*1e3,time);
     end;
  end;
  {*********}
  Icom:=Icom + Istim_ + IGJ + IC + (INa + IK + INaP + IAHP - I_Na_K_rest)*1e3;
  { ================= MODEL NEURON ================== }
  t:=time/1e3; { time step }
  Iind:=(Istim_+IGJ)/(CNrn.NP.Square*1e9);
  {******* One step of integration ******}
  if NaK.IfTurnOnModelNeuron=1 then begin
     CNrn.MembranePotential(0,0,0,0,0,Iind,0);
  end;
  {**************************************}
  Arr_Vmod[i]:=CNrn.NV.V*1000;
  { for Drawing }
  if (i_draw_NaAndK<ArrMaxNaAndK)and
     ((dt_draw=0)or(time-trunc(time/dt_draw)*dt_draw < AcqTime/2)) then begin
      i_draw_NaAndK:=i_draw_NaAndK+1;
      Arr_NaAndK[0,i_draw_NaAndK]:=time;
      Arr_NaAndK[1,i_draw_NaAndK]:=BNrn.NV.V*1e3;
      Arr_NaAndK[2,i_draw_NaAndK]:=CNrn.NV.V*1e3;
      Arr_NaAndK[3,i_draw_NaAndK]:=Icom;
      Arr_NaAndK[4,i_draw_NaAndK]:=INa;
      Arr_NaAndK[5,i_draw_NaAndK]:=IK;
      Arr_NaAndK[6,i_draw_NaAndK]:=BNrn.NV.mm;
      Arr_NaAndK[7,i_draw_NaAndK]:=BNrn.NV.hh;
      Arr_NaAndK[8,i_draw_NaAndK]:=BNrn.NV.nn;
      Arr_NaAndK[9,i_draw_NaAndK]:=INaP;
      Arr_NaAndK[10,i_draw_NaAndK]:=BNrn.NV.mNaP;
      Arr_NaAndK[11,i_draw_NaAndK]:=BNrn.NV.hNaP;
      Arr_NaAndK[12,i_draw_NaAndK]:=IC;
      Arr_NaAndK[13,i_draw_NaAndK]:=IAHP;
  end;
end;

end.
