unit AnalyzeFC;

interface
uses   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
       StdCtrls, DDSpinEdit,   TeEngine,
       Init,Init_DC,CurrentClamp,Unit1,Unit4,Unit5,Unit6,Reading,main,MathMy;

const
    MaxPulseFC=10000;
type
    vecFC   =array[0..MaxPulseFC] of double;
    vecintFC=array[0..MaxPulseFC] of integer;

    Tcoe=record
        q1,q2,k1,b1,k2,b2,   // q1*x*x + q2*y*y + k1*x*y + b1*x + k2*y + b2
        xmin,xmax,ymin,ymax, // boundaries
        xc,yc,               // center
        zmean,z2mean         // mean and second moment of z
                             :double;
    end;


    TAnalyzeFC=record
        PulseTime, PulseV, subV, VT, NegPulseDuration,
        TrueStim_I, TrueStim_G,Estim_I,Estim_G,Meander,
        Estim_GE,Estim_GI,VGABA_estimated,V_from_GE_GI,
        z1,z2,z3,PosPulseDuration,VAP,dVdt
                                                          :vecFC;
        nPulse,nFlaw,Shift,istart                         :integer;
        coe                                               :array[1..3] of Tcoe;
        dV_Flaw
                                                          :double;
    end;
var
    AFC :TAnalyzeFC;

procedure PrepareParameters_FC;
function DefineAction(nt,d1,d2,d3:integer; var IfFlaw:integer) :string;
procedure I_Meander_Offline(Action :string; var I_Meander :double);
function NumberOfAction(Action :string) :integer;
function NameOfAction(Number :integer) :string;
procedure Analyze_FC;
procedure Draw_FC;
procedure CleanCorrelates_FC;
procedure DrawCorrelations_FC;
procedure WriteCorrelations_FC(AppendOrClean :string);
procedure WriteContinuousTrueIG;
function Approximate_FC :boolean;
procedure FullAnalysisOfSelectedFiles;

implementation
uses ApproximateFC,EstimateFC,ExtracelStim,FiringClamp;


procedure PrepareParameters_FC;
begin
  FC.dnt    :=trunc(1000/FC.rate/AcqTime);  { number of steps between APs }
  FC.dntUP  :=trunc(FC.PositivePulseDuration/AcqTime);
  FC.i_end:=i_end-trunc(50{ms}/AcqTime);
  if CC.TypeOfInputIndex>=0 then Form1.ComboBox3.ItemIndex:=CC.TypeOfInputIndex
                            else CC.TypeOfInputIndex:=Form1.ComboBox3.ItemIndex;
  AFC.dV_Flaw:=Form1.DDSpinEdit13.Value;
  AFC.nFlaw:=0;
  AFC.Shift  :=trunc(Form1.DDSpinEdit23.Value);
  AFC.istart :=trunc(Form1.DDSpinEdit25.Value/AcqTime)+1;
  { Read FC parameters for meander reproduction }
//  ReadFiringClampParameters;
end;

function DefineAction(nt,d1,d2,d3:integer; var IfFlaw:integer) :string;
var I_,V_ :double;
    nt_ :integer;
    Action,Action_d3 :string;
begin
  Action   :=NameOfAction(Offline.Act[nt-1]);
  Action_d3:=NameOfAction(Offline.Act[nt-d3]);
  nt_:=nt-d2;
  I_:=VI[2,nt-d1,Smpl];
  V_:=VI[1,nt-d3,Smpl];
  if nt_<FC.dnt then Action:='Nothing' else { Skip the first spike }
  if           nt_           mod FC.dnt = 0 then begin { Positive pulse }
     if Action<>'InterPulseInterval'{='NegativePulse'} then begin
        IfFlaw:=1;
     end;
     if (I_>FC.PositivePulseCurrent/2) then begin
         Action:='SwitchToPositivePulse';
     end else begin
         Action:='DelayPositivePulse';
     end;
  end else if (Action='DelayPositivePulse')and(I_>FC.PositivePulseCurrent/2)then begin
     Action:='SwitchToPositivePulse';
  end else if (nt_-FC.dntUP) mod FC.dnt = 0 then begin { Negative pulse }
     if (I_<0) then begin
         Action:='SwitchToNegativePulse';
     end else begin
         Action:='DelayNegativePulse';
     end;
  end else if (Action='DelayNegativePulse')and(I_<0)then begin
     Action:='SwitchToNegativePulse';
  end else if (V_<Offline.FC_Vmin)and(Action='NegativePulse') then begin { Delay of meander }
{     Action:='DelayEndOfMeander';
  end else if (Action   ='DelayEndOfMeander')and
              (Action_d3='DelayEndOfMeander') then begin} { End of meander }
     Action:='EndOfMeander';
  end else begin
     if Action='SwitchToPositivePulse' then Action:='PositivePulse' else
     if Action='SwitchToNegativePulse' then Action:='NegativePulse' else
     if Action='EndOfMeander'          then Action:='InterPulseInterval';
  end;
  DefineAction:=Action;
end;

procedure I_Meander_Offline(Action :string; var I_Meander :double);
begin
  if Action='Nothing'               then I_Meander:=0 else
  if Action='DelayOfPositivePulse'  then I_Meander:=0 else
  if Action='SwitchToPositivePulse' then I_Meander:= FC.PositivePulseCurrent else
  if Action='DelayOfNegativePulse'  then I_Meander:= FC.PositivePulseCurrent else
  if Action='SwitchToNegativePulse' then I_Meander:=-FC.NegativePulseCurrent else
  if Action='DelayOfMeander'        then I_Meander:=-FC.NegativePulseCurrent else
  if Action='EndOfMeander'          then I_Meander:=0 else
  if Action='PositivePulse'         then I_Meander:= FC.PositivePulseCurrent else
  if Action='NegativePulse'         then I_Meander:=-FC.NegativePulseCurrent else
  if Action='InterPulseInterval'    then I_Meander:=0;
end;

function NumberOfAction(Action :string) :integer;
begin
  if Action='Nothing'               then NumberOfAction:=0 else
  if Action='DelayPositivePulse'    then NumberOfAction:=1 else
  if Action='SwitchToPositivePulse' then NumberOfAction:=2 else
  if Action='PositivePulse'         then NumberOfAction:=3 else
  if Action='DelayNegativePulse'    then NumberOfAction:=4 else
  if Action='SwitchToNegativePulse' then NumberOfAction:=5 else
  if Action='NegativePulse'         then NumberOfAction:=6 else
  if Action='DelayEndOfMeander'     then NumberOfAction:=7 else
  if Action='EndOfMeander'          then NumberOfAction:=8 else
  if Action='InterPulseInterval'    then NumberOfAction:=9 else
                                         NumberOfAction:=0;
end;

function NameOfAction(Number :integer) :string;
begin
  case Number of
  0: NameOfAction:='Nothing';
  1: NameOfAction:='DelayPositivePulse';
  2: NameOfAction:='SwitchToPositivePulse';
  3: NameOfAction:='PositivePulse';
  4: NameOfAction:='DelayNegativePulse';
  5: NameOfAction:='SwitchToNegativePulse';
  6: NameOfAction:='NegativePulse';
  7: NameOfAction:='DelayEndOfMeander';
  8: NameOfAction:='EndOfMeander';
  9: NameOfAction:='InterPulseInterval';
  end;
end;

procedure Correct_Junction_Offset_Ra_and_Ireal(nt_:integer; V_,Ireal_:double;
                                                            var I_,G_:double);
var dV_Junction_,G1_,dI_offset_,Vm_ :double;
begin
  dV_Junction_:=Offline.dV_Junction; {mV}   // Junction potential
  G1_         :=Offline.G1;          {nS}   // Series conductance, G1=1/Ra
  dI_offset_  :=Offline.dI_offset;   {pA}   // Offset current - additional polarization
  { Recreate meander }
  i:=nt_;    // global time step
  V:=V_;     // global voltage as during the recordings
  RealTime_FiringClamp;
  {*******************}
  if (G1_>0.01{nS})and(G_>0) then begin  // compensate series resistance
     Vm_:=V_ - dV_Junction_ - (Ireal_-FC.Meander)/G1_;
     G_:=G_/(1+G_/G1_);
     I_:=I_ + dI_offset_ - G_*dV_Junction_ - (I_+FC.Meander+dI_offset_)*G_/G1_;
//     I_ :=Ireal_ - FC.Meander + G_*(Vm_-Vus)*(1-G_/G1_);
  end else begin
     Vm_:=V_-dV_Junction_;
     I_ :=I_ + dI_offset_ - G_*dV_Junction_;
//     I_ :=Ireal_ - FC.Meander + G_*(Vm_-Vus);
  end;
end;

{******************************************************************************}
procedure Analyze_FC;
var
    nt_,iPulse,nS,nSIG,nSM,IfFlaw                 :integer;
    V_,Ireal_,SV,SI,SG,SM,I_,G_,C_,Vprev,
    Vmax_,Vmin_,tEofM
                                                  :double;
    Action                                        :string;
begin
  PrepareParameters_FC;
  { Initial parameters }
  IfFlaw:=0; nS:=0; nSIG:=0; nSM:=0;
  Vmax_:=Offline.Vm[AFC.istart];  Vmin_:=Offline.Vm[AFC.istart-1];
  SV   :=Offline.Vm[AFC.istart];
  SI   :=Offline.I[AFC.istart];   SG:=Offline.G[AFC.istart];
  SM   :=Offline.Meander[AFC.istart];
  tEofM:=dt*(nt_-1)*1000;
  iPulse:=0;
  { Loop for time-steps }
  for nt_:=AFC.istart to imin(Nexp[Smpl]-3,FC.i_end) do begin
      t:=dt*nt_*1000;
      V_    :=Offline.Vm[nt_];
      Ireal_:=Offline.Ireal[nt_];
      Vprev :=Offline.Vm[nt_-1];
      { Check for a flaw }
      if (abs(V_-Vprev)>AFC.dV_Flaw)or(abs(V_)>=150{mV}) then begin
          IfFlaw:=1;
      end;
      { Define Action }
      Action:=NameOfAction(Offline.Act[nt_]);
      if Offline.IfFlaw[nt_]=1 then IfFlaw:=1;
      { Define True Stimul }
      I_:=Offline.I[nt_];
      G_:=Offline.G[nt_];
      { correct Vus=-70mV, not 0 }
      if Form1.CheckBox16.Checked then  I_:=I_-G_*(-70{mV});
      { Measure }
      if          Action='SwitchToPositivePulse' then begin
         AFC.dVdt[iPulse]:=(V_-Vmin_)/(t-tEofM);
         AFC.VT[iPulse]:=min(V_,Vprev);
         AFC.subV[iPulse]:=SV;
         AFC.PosPulseDuration[iPulse]:=t;
         Vmax_:=V_;
      end else if (Action='PositivePulse')or(Action='DelayNegativePulse') then begin
         Vmax_:=max(Vmax_,V_);
      end else if Action='SwitchToNegativePulse' then begin
         AFC.PulseTime[iPulse] :=t;
         AFC.PulseV[iPulse]    :=max(Vmax_,V_);
         AFC.PosPulseDuration[iPulse]:=t-AFC.PosPulseDuration[iPulse];
         AFC.VAP[iPulse]       :=AFC.PulseV[iPulse]-AFC.VT[iPulse];
         if AFC.PosPulseDuration[iPulse]>0 then AFC.VAP[iPulse]:=AFC.VAP[iPulse]
           /AFC.PosPulseDuration[iPulse]*FC.PositivePulseDuration;
         AFC.TrueStim_I[iPulse]:=SI;
         AFC.TrueStim_G[iPulse]:=SG;
         AFC.Meander[iPulse]   :=SM;
         nSIG:=0; nSM:=0;
         SI:=I_; SG:=G_;
         AFC.NegPulseDuration[iPulse]:=t;
      end else if Action='EndOfMeander' then begin
         AFC.NegPulseDuration[iPulse]:=t-AFC.NegPulseDuration[iPulse];
         tEofM:=t;    Vmin_:=V_;
         SV:=0; nS:=0;
         if IfFlaw=1 then begin
            iPulse:=iPulse-1; AFC.nFlaw:=AFC.nFlaw+1; IfFlaw:=0;
         end;
         iPulse:=iPulse+1;
      end else if Action='InterPulseInterval' then begin
         nS:=nS+1;
         SV:=(SV*(nS-1)   + V_)/nS;
         nSIG:=nSIG+1;
//         SI:=(SI*(nSIG-1) + I_)/nSIG;
//         SG:=(SG*(nSIG-1) + G_)/nSIG;
         SI:=SI+dt*1000/0.5{ms}*(-SI+I_);
         SG:=SG+dt*1000/0.5{ms}*(-SG+G_);
      end;
      nSM:=nSM+1;
      SM:=(SM*(nSM-1) + Offline.Meander[nt_])/nSM;
  end;
  AFC.nPulse:=iPulse-2;
end;
{******************************************************************************}

procedure Draw_FC;
var
    i,iPulse,iStim,skip       :integer;
    tP,I_,G_,C_,t_Stim        :double;
    t1                        :string;
    IfAlignToStimuli          :boolean;
begin
  IfAlignToStimuli:=Form1.CheckBox14.Checked;
  Form1.LineSeries1.Clear;
  Form6.LineSeries3.Clear;
  Form6.Series1.Clear;   //Stimuli
  Form6.Series2.Clear;   //Iextract
  Form5.Series5.Clear;
  Form5.Series6.Clear;
  Form1.Series30.Clear;
  skip:=trunc(Form1.DDSpinEdit3.Value);
  for i:=0 to Nexp[Smpl] do if ((i=0)or(i mod skip=0))and(i<nDrawMax*skip) then begin
      { Define True Stimul }
      t_Stim:=dt*i*1000;
      { Align to stimuli }
      if IfAlignToStimuli then begin
         NumberOfStimul(t_Stim,iStim);
      end;
      Form1.LineSeries1.AddXY(t_Stim,Offline.Vm[i]);
      if i mod 10=0 then begin
         Form6.LineSeries3.AddXY(t_Stim,VI[2,i,Smpl]);
         Form5.Series5.AddXY(t_Stim,Offline.I[i]);
         Form5.Series6.AddXY(t_Stim,Offline.G[i]);
      end;
      if Offline.Markers[i]=1 then
         Form1.Series30.AddXY(t_Stim,Offline.Vm[i]);
  end;
  Application.ProcessMessages;
  { Plot mesured values }
  Form1.PointSeries2.Clear;
  Form1.PointSeries3.Clear;
  Form1.PointSeries4.Clear;
  Form1.PointSeries5.Clear;
  Form1.Series29.Clear;
  Form6.PointSeries6.Clear;
  for iPulse:=1 to AFC.nPulse do begin
      tP:=AFC.PulseTime[iPulse];
      i:=trunc(AFC.PulseTime[iPulse]/1000/dt);
      { Align to stimuli }
      if IfAlignToStimuli then NumberOfStimul(tP,iStim);
      Form1.PointSeries2.AddXY(tP,AFC.PulseV[iPulse]);
      Form1.PointSeries3.AddXY(tP,AFC.subV[iPulse]);
      Form1.PointSeries4.AddXY(tP,AFC.NegPulseDuration[iPulse]);
      Form1.PointSeries5.AddXY(tP,AFC.VT[iPulse]);
      Form6.PointSeries6.AddXY(tP,VI[2,i,Smpl]);
      Form1.Series29.AddXY(tP,AFC.dVdt[iPulse]);
  end;
  str(AFC.nFlaw:3,t1);
  Form1.Label22.Caption:='Number of flaws='+t1;
  Application.ProcessMessages;
end;

procedure CleanCorrelates_FC;
begin
  Form1.PointSeries7.Clear;
  Form1.PointSeries8.Clear;
  Form1.Series12.Clear;
  Form1.Series13.Clear;
  Form1.PointSeries1.Clear;
  Form1.PointSeries9.Clear;
  Form1.PointSeries10.Clear;
  Form1.PointSeries11.Clear;
  Form1.Series14.Clear;
  Form1.Series15.Clear;
  Form1.Series27.Clear;
  Form1.Series28.Clear;
end;

procedure DrawCorrelations_FC;
{ Plot measured values versus true I and G }
var
    iPulse                    :integer;
    tP                        :double;
begin
  { Plot mesured values }
  for iPulse:=1 to AFC.nPulse do begin
      tP:=AFC.PulseTime[iPulse];
      { Left Chart }
      if Form1.CheckBox1.Checked then
      Form1.PointSeries7. AddXY(AFC.TrueStim_I[iPulse],AFC.subV[iPulse]);
      if Form1.CheckBox2.Checked then
      Form1.PointSeries8. AddXY(AFC.TrueStim_I[iPulse],AFC.NegPulseDuration[iPulse]);
      if Form1.CheckBox3.Checked then
      Form1.Series12.AddXY(AFC.TrueStim_I[iPulse],AFC.VT[iPulse]);
      if Form1.CheckBox4.Checked then
      Form1.Series13.AddXY(AFC.TrueStim_I[iPulse],AFC.VAP[iPulse]);
      if Form1.CheckBox5.Checked then
      Form1.Series14.AddXY(AFC.TrueStim_I[iPulse],AFC.PulseV[iPulse]);
      if Form1.CheckBox22.Checked then
      Form1.Series27.AddXY(AFC.TrueStim_I[iPulse],AFC.dVdt[iPulse]);
      { Right Chart }
      if Form1.CheckBox6.Checked then
      Form1.PointSeries1. AddXY(AFC.TrueStim_G[iPulse],AFC.subV[iPulse]);
      if Form1.CheckBox7.Checked then
      Form1.PointSeries9. AddXY(AFC.TrueStim_G[iPulse],AFC.NegPulseDuration[iPulse]);
      if Form1.CheckBox8.Checked then
      Form1.PointSeries10.AddXY(AFC.TrueStim_G[iPulse],AFC.VT[iPulse]);
      if Form1.CheckBox9.Checked then
      Form1.PointSeries11.AddXY(AFC.TrueStim_G[iPulse],AFC.VAP[iPulse]);
      if Form1.CheckBox10.Checked then
      Form1.Series15.AddXY(AFC.TrueStim_G[iPulse],AFC.PulseV[iPulse]);
      if Form1.CheckBox23.Checked then
      Form1.Series28.AddXY(AFC.TrueStim_G[iPulse],AFC.dVdt[iPulse]);
  end;
  Application.ProcessMessages;
end;

procedure WriteCorrelations_FC(AppendOrClean :string);
{ Write measured values versus true I and G }
var
    iPulse                    :integer;
    tP                        :double;
    bbb                       :text;
    FName                     :string;
begin
  FName:='I_G_VT_dV_subV_PulseV_PulseDur.dat';
  assign(bbb,FName);
  if FileExists(FName) then begin
     append(bbb)
  end else begin
     rewrite(bbb);
  end;
  if AppendOrClean='Clean' then begin
     rewrite(bbb); close(bbb);
     DeleteFile(FName);      // 2.03.2013
     exit;
  end;
  { Plot mesured values }
  for iPulse:=1 to AFC.nPulse do begin
      tP:=AFC.PulseTime[iPulse];
      write  (bbb,AFC.TrueStim_I[iPulse]:8:3,' ',AFC.TrueStim_G[iPulse]:8:3,' ');
      write  (bbb,AFC.VT[iPulse]:8:3,' ',AFC.VAP[iPulse]:8:3,' ');
      write  (bbb,AFC.subV[iPulse]:8:3,' ',AFC.PulseV[iPulse]:8:3,' ');
      write  (bbb,AFC.NegPulseDuration[iPulse]:8:3,' ',AFC.dVdt[iPulse]:8:3,' ');
      writeln(bbb,tP:8:3);
  end;
  if FileSize(bbb)<=0 then begin close(bbb); DeleteFile(FName); exit; end; // 2.03.2013
  close(bbb);
end;

procedure WriteContinuousTrueIG;
var
    i                         :integer;
    I_,G_,C_                  :double;
    bbb                       :text;
    FName                     :string;
begin
  FName:='True_I_G(t).dat';
  assign(bbb,FName);  rewrite(bbb);
  for i:=0 to Nexp[Smpl] do begin
      if i mod 10 = 0 then begin
         { Define True Stimul }
         Define_I_G(dt*(i-2)*1000, I_,G_,C_);
         writeln(bbb,dt*i*1000:8:3,' ',I_:8:3,' ',G_:8:3);
      end;
  end;
  close(bbb);
end;

function Approximate_FC :boolean;
var ic1,ic2,ic3 :integer;
    t1,t2,s1,s2,s3,s4   :string;
    bbb :text;
begin
  Approximate_FC:=false;
  { Which column to read from file: }
  { 3-VT, 4-PulseV-VT, 5-subV, 6-PulseV, 7-NegPulseDuration, 8-dVdt }
  ic1:=trunc(Form1.DDSpinEdit9.Value);
  ic2:=trunc(Form1.DDSpinEdit10.Value);
  ic3:=trunc(Form1.DDSpinEdit30.Value);
  {*************************************}
  if Approximate_FC_for_raw(ic1, AFC.coe[1])=false then exit;
  if Approximate_FC_for_raw(ic2, AFC.coe[2])=false then exit;
  if Approximate_FC_for_raw(ic3, AFC.coe[3])=false then exit;
  { Writing down the formula }
  str(AFC.coe[1].k1:7:4,s1);  str(AFC.coe[1].b1:7:4,s2);
  str(AFC.coe[1].k2:6:3,s3);  str(AFC.coe[1].b2:7:3,s4);
  t1:='z1=('+s1+')*x*y+('+s2+')*x+('+s3+')*y+('+s4+')';
  str(AFC.coe[2].k1:7:4,s1);  str(AFC.coe[2].b1:7:4,s2);
  str(AFC.coe[2].k2:6:3,s3);  str(AFC.coe[2].b2:7:3,s4);
  t2:='z2=('+s1+')*x*y+('+s2+')*x+('+s3+')*y+('+s4+')';
  Form1.Label24.Caption:=t1;
  Form1.Label25.Caption:=t2;
  { Write eqs. in file }
  assign(bbb,'Equations.txt'); rewrite(bbb);
  writeln(bbb,t1);
  writeln(bbb,t2);
  close(bbb);
  { Open form for Estimation }
  Form5.Visible:=true;  {Form5: Estimations_FC}
  Approximate_FC:=true;
end;

procedure FullAnalysisOfSelectedFiles;
begin
  Form1.Label19.Caption:='Read selected files';
  {  *************************  }
  if ReadAndTreatSelectedFiles=false then exit;
  {  *************************  }
  { Choose proper page }
  if          (If_FiringClamp=1)and(Form1.PageControl1.ActivePageIndex=0) then begin
      Form1.PageControl1.ActivePageIndex:=3;
  end else if (If_FiringClamp=0)and(Form1.PageControl1.ActivePageIndex=3) then begin
      Form1.PageControl1.ActivePageIndex:=0;
  end;
  { Repeat reading for FC }
  Smpl:=0;
  REPEAT Smpl:=Smpl+1;
    ReadPastTrace(FNames_Umea[Smpl]);
    { Firing-Clamp }
    if (Form1.PageControl1.ActivePageIndex in [3,4])and(If_FiringClamp=1) then begin
          Form1.Label19.Caption:='Analyze_FC';
          Analyze_FC;
          Form1.Label19.Caption:='Draw_FC';
          Draw_FC;
          Form1.Label19.Caption:='Draw and write correlations';
          DrawCorrelations_FC;
          if Form1.CheckBox11.Checked then begin //Write in file
             WriteCorrelations_FC('Append');
          end;
          Form1.Label19.Caption:='No Approximation';
          if Form1.CheckBox12.Checked then begin //Approximate
             Form1.Label19.Caption:='Approximation';
             if Approximate_FC=false then exit;
          end;
          Form1.Label19.Caption:='No Estimation';
          if (Form5.Visible)and(Form1.CheckBox12.Checked) then begin
             Form1.Label19.Caption:='Estimation';
             Estimate_FC;
             AverageEstimatedPoints;
             WriteContinuousTrueIG;
          end;
          Form1.Label19.Caption:='Done';
    end;
  UNTIL Smpl=NSmpl;
end;

end.
