unit Analyzer;

interface
uses   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
       StdCtrls, DDSpinEdit,
       Unit1,Unit3,Init,MathMy,ExtracelStim;

procedure FindAllSpikes(Smpl :integer);
procedure SuperimposeV;

var
    ISI,tAPs,Vav,DVav,WidthAP,Vpeak,VT,VAHP,VAHPfast,tAP_th        :vectAP;
    IIspike                                                        :integer;

implementation
uses AnalyzeFC,Init_DC;

procedure IfSpikeOccurs(t_,V_,Vprev_,Vpprev_ :double; var tAP_:double; var indic_:integer);
var ddV,dVdt :double;
{Defines if a spike occurs for V_.
 'indic_' = 0 - after spike
            1 - on ascending way
            2 - at spike
 'Vprev_' - potential at previous time step}
begin
  if (indic_=2)or(t_<tAP_+0.001) then indic_:=0;  { after spike }
  if (indic_<>1)and(V_<SpikeCriteria.V_min) then indic_:=3;  { recovered state }
  if ((indic_=3)or(indic_=1))and(V_>SpikeCriteria.V_min{-35mV}) then begin
     { Find local maximum }
     dVdt:=min((V_-Vprev_)/dt,(V_-Vpprev_)/2/dt)/1000 {mV/ms};
     //min_dVdt:=Form1.DDSpinEdit11.Value{20mV/ms};
     if dVdt>SpikeCriteria.dVdt_min then begin
        indic_:=1;            { on ascending way }
     end else begin
         ddV:=(V-2*Vprev_+Vpprev_)/dt/dt/1e6 {mV/ms^2};
         //min_d2Vdt2:=Form1.DDSpinEdit12.Value{10 mV/ms^2};
         if (indic_=1)and(ddV<-SpikeCriteria.d2Vdt2_min) then begin
            indic_:=2;        { at spike }
            //ISI[1]:=min(t-tAP_,t-tStartClamp{idelay});
            tAP_:=t_;
         end;
     end;
     //Vpprev_:=Vprev_;
     //Vprev_ :=V_;
  end;
end;

procedure FindThreshold(nt_e,Smpl,IIspike:integer);
var nStepBack,nt_,nt_b,nt_max          :integer;
    dVdt1,dVdt2,ddV,dV,K_max,K_,sc     :double;
begin
  sc:=Form1.DDSpinEdit20.Value; {mV/ms}
  nStepBack:=trunc(0.008/dt);
  nt_b:=imax(trunc(tStartClamp/1000/dt)+5,nt_e-nStepBack);
  K_max:=0; nt_max:=nt_b;
  dV :=(VI[1,nt_b+1,Smpl]-VI[1,nt_b,Smpl])/(dt*1000);
  ddV:=(VI[1,nt_b+1,Smpl]-2*VI[1,nt_b,Smpl]+VI[1,nt_b-1,Smpl])/sqr(dt*1000);
  for nt_:=nt_b to nt_e do begin
//      ddV:=minmod((VI[1,nt_+4,Smpl]-2*VI[1,nt_,Smpl]+VI[1,nt_-4,Smpl])/sqr(4*dt*1000),
//                  (VI[1,nt_+2,Smpl]-2*VI[1,nt_,Smpl]+VI[1,nt_-2,Smpl])/sqr(2*dt*1000)); {mV/ms^2}
      ddV:=ddV+dt*1000/0.1{ms}*(-ddV+(VI[1,nt_+1,Smpl]-2*VI[1,nt_,Smpl]+VI[1,nt_-1,Smpl])/sqr(dt*1000));
      dV :=dV +dt*1000/0.5{ms}*(-dV +(VI[1,nt_+1,Smpl]-VI[1,nt_,Smpl])/(dt*1000));
//      dV :=max(max((VI[1,nt_+4,Smpl]-VI[1,nt_-4,Smpl])/( 8*dt*1000),
//                   (VI[1,nt_+8,Smpl]-VI[1,nt_-8,Smpl])/(16*dt*1000)),
//                   (VI[1,nt_+2,Smpl]-VI[1,nt_-2,Smpl])/( 4*dt*1000)); {mV/ms}
//      ddV2:=(VI[1,nt_+2,Smpl]-2*VI[1,nt_+1,Smpl]+VI[1,nt_,Smpl])/sqr(dt*1000); {mV^2/ms^2}
      K_:=ddV/step(sqr(sc)+sqr(dV),1.5);      // K=abs(f'')/(1+f'^2)^1.5 - curvature of y=f(x) [http://mathem.h1.ru/diff13.html]
      if (dV>1e-6)and(K_>K_max) then begin
         K_max:=K_;
         nt_max:=nt_;
      end;
  end;
  VT[IIspike]:=VI[1,nt_max-1,Smpl];
  tAP_th[IIspike]:=(nt_max-1)*dt;
//  nt:=nt_b;
//  repeat nt_:=nt_+1;
//      dVdt1:=(VI[1,nt_,Smpl]  -VI[1,nt_-1,Smpl])/dt/1000; {mV/ms}
//      dVdt2:=(VI[1,nt_-1,Smpl]-VI[1,nt_-2,Smpl])/dt/1000; {mV/ms}
//  until (nt_=nt0)or(min(dVdt1,dVdt2)>SpikeCriteria.dVdt_min{20mV/ms});
//  VT[IIspike]:=VI[1,nt_-1,Smpl];
//  tAP_th[IIspike]:=(nt_-1)*dt;
end;

procedure MeasureSpikeShape(Smpl :integer);
var N_,nt_,nt0,indic2_,IfFirstSpikeHappened               :integer;
    V_,VAHP_,VT_,Vpeak_,V12,tAP1,tAP2,dum,dVdt_,tpeak_,t_ :double;
begin
  nt0:=trunc(tAP_th[IIspike]/dt);
  N_ :=trunc(SpikeCriteria.TimeWindow/dt);
  IfFirstSpikeHappened:=0;
  { VT, Vpeak and VT }
  VT_:=VI[1,nt0,Smpl];
  Vpeak_:=VT_;
  nt_:=nt0; indic2_:=1; dum:=-888;
  repeat nt_:=nt_+1;  t_:=nt_*dt;
      if Vpeak_<VI[1,nt_,Smpl] then begin
         Vpeak_:=VI[1,nt_,Smpl];
         tpeak_:=t_;
      end;
      { Check if the second spike appears }
      IfSpikeOccurs(t_,VI[1,nt_,Smpl],VI[1,nt_-1,Smpl],VI[1,nt_-2,Smpl],dum,indic2_);
      if indic2_=2 then IfFirstSpikeHappened:=1;
  until (nt_=nt0+N_) or ((indic2_=1)and(IfFirstSpikeHappened=1))
                     or (t_>tEndClamp/1000);
  N_:=nt_-nt0;
  Vpeak[IIspike]:=Vpeak_;
  tAPs[IIspike]:=tpeak_;
  V12:=(Vpeak_+VT_)/2;
  { Width of AP }
  tAP1:=0; tAP2:=0;
  nt_:=nt0; indic2_:=1; dum:=-888;
  repeat nt_:=nt_+1;
      if (VI[1,nt_+1,Smpl]>=V12)and(VI[1,nt_,Smpl]<V12) then begin
         tAP1:=(nt_-1)*dt;
         if abs(VI[1,nt_,Smpl]-VI[1,nt_-1,Smpl])>1e-6 then
         tAP1:=tAP1+(V12-VI[1,nt_-1,Smpl])/(VI[1,nt_,Smpl]-VI[1,nt_-1,Smpl])*dt;
      end else if (VI[1,nt_+1,Smpl]<=V12)and(VI[1,nt_,Smpl]>V12) then begin
         tAP2:=(nt_-1)*dt;
         if abs(VI[1,nt_,Smpl]-VI[1,nt_-1,Smpl])>1e-6 then
         tAP2:=tAP2+(V12-VI[1,nt_-1,Smpl])/(VI[1,nt_,Smpl]-VI[1,nt_-1,Smpl])*dt;
         WidthAP[IIspike]:=(tAP2-tAP1)*1000;
      end;
  until (nt_=nt0+N_);
  { VAHP }
  VAHP_:=Vpeak_;
  nt_:=nt0; indic2_:=0; dum:=-888;
  repeat nt_:=nt_+1;
      if (nt_*dt>tAP2)and(VI[1,nt_,Smpl]<VAHP_) then VAHP_:=VI[1,nt_,Smpl];
  until (nt_=nt0+N_);
  VAHP[IIspike]:=VAHP_;
  { VAHP-fast }
  nt_:=nt0; dVdt_:=-1e6;
  repeat nt_:=nt_+1;
      if (nt_*dt>tAPs[IIspike]+AHPfast.TimeSincePeak)then
         dVdt_:=minmod(VI[1,nt_+1,Smpl]-VI[1,nt_,Smpl],
                       VI[1,nt_,Smpl]  -VI[1,nt_-1,Smpl])/(dt*1e3);
  until  (nt_=nt0+N_)or(dVdt_>AHPfast.dVdt_min);
  VAHPfast[IIspike]:=VI[1,nt_,Smpl];
end;

procedure FindAllSpikes(Smpl :integer);
var
        Vprev,Vpprev,tAP,topV,dVdt,tAP1,tAP2,t_ :double;
        i,nt_,indic,n_av,iStim                  :integer;
begin
  if If_FiringClamp=1 then begin
//          Analyze_FC;
//          Draw_FC;
          exit;
  end;
//  Form1.Series6.Clear;
  t:=0;
  for i:=0 to mMaxAP do tAPs[i]:=0;       tAPs[0]:=tStartClamp/1000;
  for i:=0 to mMaxAP do ISI[i]:=0;
  for i:=0 to mMaxAP do WidthAP[i]:=0;    tAP1:=0; tAP2:=0;
  for i:=0 to mMaxAP do Vpeak[i]:=0;
  for i:=0 to mMaxAP do VT[i]:=0;
  tAP:=-888;
  IIspike:=0;
  for nt_:={trunc(tStartClamp/1000/dt)}+1 to imin(trunc(tEndClamp/1000/dt),Nexp[Smpl]) do begin
      t:=dt*nt_;
      V:=VI[1,nt_,Smpl]; Vprev:=VI[1,nt_-1,Smpl];  Vpprev:=VI[1,nt_-2,Smpl];
      IfSpikeOccurs(t,V,Vprev,Vpprev,tAP,indic);
      { if spike }
      if (indic=2){and(t>tStartClamp/1000)} then begin
                                                          {*****************}
         if (t>tStartClamp/1000)and(IISpike<mMaxAP) then  IIspike:=IIspike+1;
                                                          {*****************}
//         tAPs[IIspike]:=t;
//         Vpeak[IIspike]:=VI[1,nt_-1,Smpl];
         n_av:=0;
//         Form1.Series6.AddXY(tAPs[IIspike]*1000,-40);
         { Calculate ISI }
         if (IIspike<=mMaxAP)and(IIspike>0) then begin
            ISI[IIspike]:=t-tStartClamp{idelay}/1000;
            if IIspike>1 then
               for i:=1 to IIspike-1 do
                   ISI[IIspike]:=ISI[IIspike]-ISI[i];
         end;
         { Find threshold }
         FindThreshold(nt_,Smpl,IIspike);
         MeasureSpikeShape(Smpl);
//         Application.ProcessMessages;
      end;
      { Mean voltage between the spikes }
      if indic=2 then begin Vav[IIspike]:=0; DVav[IIspike]:=0; end;
      if (indic<>2)and(t-tAP>0) then begin
          topV:=-15{mV};                          {!!!}
          { Average potential }
          dVdt:=(V-VI[1,nt_-1,Smpl])/dt/1000; {mV/ms}
          if (V<topV)and(t-tAP>0.002)and(abs(dVdt)<5) then begin
             n_av:=n_av+1;
             Vav [IIspike]:=( Vav[IIspike]*(n_av-1) + V)/n_av;
             DVav[IIspike]:=(DVav[IIspike]*(n_av-1) + sqrt(abs(V-Vprev))/dt)/n_av;
//             Form1.Series5.AddXY(dt*nt_*1000,VI[1,nt_,Smpl]);
//             Application.ProcessMessages;
          end;
      end;
      { Width of AP }
//      if (indic=1)and(V>-20{mV})and(VI[1,nt_-1,Smpl]<-20{mV}) then begin
//          tAP1:=(nt_-1)*dt;
//          tAP1:=tAP1+(-20{mV}-VI[1,nt_-1,Smpl])/(V-VI[1,nt_-1,Smpl])*dt;
//      end;
//      if (indic=0)and(V<-20{mV})and(VI[1,nt_-1,Smpl]>-20{mV}) then begin
//          tAP2:=(nt_-1)*dt;
//          tAP2:=tAP2+(-20{mV}-VI[1,nt_-1,Smpl])/(V-VI[1,nt_-1,Smpl])*dt;
//          WidthAP[IIspike]:=(tAP2-tAP1)*1000;
//      end;
  end;
  { Drawing }
  Form1.Series6.Clear;
  Form1.Series7.Clear;
  Form1.Series8.Clear;
  Form1.Series9.Clear;
  Form1.Series20.Clear;
  Form1.Series21.Clear;
  for i:=1 to IIspike do begin
      t_:=tAPs[i]*1000;
      { Align to stimuli }
      if Form1.CheckBox15.Checked then NumberOfStimul(t_,iStim);
      { Draw Spikes }
      Form1.Series6.AddXY(t_,Vpeak[i]);
      { Draw Mean Voltage }
      Form1.Series7.AddXY(t_-ISI[i]/2*1000,Vav[i]);
      { Draw WidthAP }
      Form1.Series8.AddXY(t_,WidthAP[i]);
      { Draw VT }
      Form1.Series9.AddXY(tAP_th[i]*1e3,VT[i]);
      { Draw VAHP }
      Form1.Series20.AddXY(t_,VAHP[i]);
      { Draw VAHPfast }
      Form1.Series21.AddXY(t_,VAHPfast[i]);
  end;
  Application.ProcessMessages;
end;

procedure SuperimposeV;
var nt_,iSp     :integer;
    t0 :double;
begin
  Form3.Visible:=true;
  Form3.Series1.Clear;
  Smpl:=trunc(Form1.DDSpinEdit2.Value);
  FindAllSpikes(Smpl);
  for nt_:=1 to Nexp[Smpl] do begin
      t:=dt*nt_;
      { Number of the previous spike }
      iSp:=0;
      repeat iSp:=iSp+1;
      until t<tAPs[iSp];
      iSp:=iSp-1;
      t0:=0; if iSp>0 then t0:=tAPs[iSp];
      { Draw Voltage }
      Form3.Series1.AddXY((t-t0)*1000,VI[1,nt_,Smpl]);
  end;
  Application.ProcessMessages;
end;

end.
