unit ApproximateFC;

interface
uses   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
       StdCtrls, DDSpinEdit, Math,
       Init,Unit1,Unit7,AnalyzeFC,MathMy,FrameLattice;

function z_of_x_y(x,y :double; coe :Tcoe) :double;
function dzdx_of_x_y(x,y :double; coe :Tcoe) :double;
function dzdy_of_x_y(x,y :double; coe :Tcoe) :double;
function ddzdxx_of_x_y(x,y :double; coe :Tcoe) :double;
function ddzdxy_of_x_y(x,y :double; coe :Tcoe) :double;
function ddzdyy_of_x_y(x,y :double; coe :Tcoe) :double;
procedure Approximate_FC_for_raw_cylinder(ic :integer; var coe :Tcoe);
procedure Approximate_FC_for_raw_linear(ic :integer; var coe :Tcoe);
procedure Approximate_FC_for_raw_quadratic(ic :integer; var coe :Tcoe);
function Approximate_FC_for_raw(ic :integer; var coe :Tcoe):boolean;
procedure NearestInCalibrationSet(z1,z2,z3 :double; var x,y :double);
procedure DrawApproximation1_FC(ic :integer; coe :Tcoe);
procedure DrawApproximation2_FC(ic :integer; coe :Tcoe);
procedure DrawLattices(I_min,I_max,G_min,G_max :double);
procedure WriteLatticesAndCalibrationPoints;

type
  TAmplifierSatuation = record
    dI :array[3..7] of double; // shift due to saturation
    Gscale_Of_Sigmoid :double; // scale of sigmoid function of the saturation
  end;
var AmplSat :TAmplifierSatuation;

implementation

var
    Np                        :integer;      // number of calibration points
    V1,V2,V3,V4,V5,V6,V7,V8   :vecFC;

function z_of_x_y_inset(x,y :double; coe :Tcoe) :double;
begin
  z_of_x_y_inset:= coe.q1*x*x + coe.q2*y*y
                 + coe.k1*x*y + coe.b1*x + coe.k2*y + coe.b2;
end;

function z_of_x_y(x,y :double; coe :Tcoe) :double;
{ Use quadratic interpolation and linear extrapolation }
var xc,yc,zc,fi,xs,ys,zs,r,rs,r1,r2,r3,r4,fi1,fi2,fi3,fi4 :double;
begin
  if (x>=coe.xmin)and(x<=coe.xmax)and(y>=coe.ymin)and(y<=coe.ymax) then begin
      z_of_x_y:=z_of_x_y_inset(x,y,coe);
  end else begin
      { Find center of rectangle }
      xc:=(coe.xmin+coe.xmax)/2;
      yc:=(coe.ymin+coe.ymax)/2;
      r:=sqrt(sqr(x-xc)+sqr(y-yc));
      zc:=z_of_x_y_inset(xc,yc,coe);
      { Find a ray from center to (x,y) }
      fi:=ArcSin((y-yc)/r);  if x<xc then fi:=pi-fi;
      r1:=sqrt(sqr(coe.xmax-xc)+sqr(coe.ymax-yc));
      fi1:=ArcSin((coe.ymax-yc)/r1);  if coe.xmax<xc then fi1:=pi-fi1;
      r2:=sqrt(sqr(coe.xmin-xc)+sqr(coe.ymax-yc));
      fi2:=ArcSin((coe.ymax-yc)/r2);  if coe.xmin<xc then fi2:=pi-fi2;
      r3:=sqrt(sqr(coe.xmin-xc)+sqr(coe.ymin-yc));
      fi3:=ArcSin((coe.ymin-yc)/r3);  if coe.xmin<xc then fi3:=pi-fi3;
      r4:=sqrt(sqr(coe.xmax-xc)+sqr(coe.ymin-yc));
      fi4:=ArcSin((coe.ymin-yc)/r4);  if coe.xmax<xc then fi4:=pi-fi4;
      { Find a cross-point of the rectangle and the ray }
      if          (fi>fi4)and(fi<=fi1) then begin       {right}
         xs:=coe.xmax;
         ys:=tan(fi)*(xs-xc)+yc;
      end else if (fi>fi1)and(fi<=fi2) then begin       {top}
         ys:=coe.ymax;
         if tan(fi)<1e8 then xs:=(ys-yc)/tan(fi)+xc else xs:=xc;
      end else if (fi>fi2)and(fi<=fi3) then begin       {left}
         xs:=coe.xmin;
         ys:=tan(fi)*(xs-xc)+yc;
      end else begin                                    {bottom}
         ys:=coe.ymin;
         if tan(fi)<1e8 then xs:=(ys-yc)/tan(fi)+xc else xs:=xc;
      end;
      zs:=z_of_x_y_inset(xs,ys,coe);
      rs:=sqrt(sqr(xs-xc)+sqr(ys-yc));
      { Linear extrapolation }
      z_of_x_y:=zc+(zs-zc)/rs*r;
  end;
end;

function dzdx_of_x_y(x,y :double; coe :Tcoe) :double;
var dx :double;
begin
  if (x>=coe.xmin)and(x<=coe.xmax)and(y>=coe.ymin)and(y<=coe.ymax) then begin
      dzdx_of_x_y:=2*coe.q1*x + coe.k1*y + coe.b1;
  end else begin
      dx:=(coe.xmax-coe.xmin)/1000;
      dzdx_of_x_y:=(z_of_x_y(x+dx,y,coe)-z_of_x_y(x,y,coe))/dx;
  end;
end;

function dzdy_of_x_y(x,y :double; coe :Tcoe) :double;
var dy :double;
begin
  if (x>=coe.xmin)and(x<=coe.xmax)and(y>=coe.ymin)and(y<=coe.ymax) then begin
      dzdy_of_x_y:=2*coe.q2*y + coe.k1*x + coe.k2;
  end else begin
      dy:=(coe.ymax-coe.ymin)/1000;
      dzdy_of_x_y:=(z_of_x_y(x,y+dy,coe)-z_of_x_y(x,y,coe))/dy;
  end;
end;

function ddzdxx_of_x_y(x,y :double; coe :Tcoe) :double;
begin
  if (x>=coe.xmin)and(x<=coe.xmax)and(y>=coe.ymin)and(y<=coe.ymax) then begin
      ddzdxx_of_x_y:=2*coe.q1;
  end else begin
      ddzdxx_of_x_y:=0;
  end;
end;

function ddzdxy_of_x_y(x,y :double; coe :Tcoe) :double;
begin
  if (x>=coe.xmin)and(x<=coe.xmax)and(y>=coe.ymin)and(y<=coe.ymax) then begin
      ddzdxy_of_x_y:=coe.k1;
  end else begin
      ddzdxy_of_x_y:=0;
  end;
end;

function ddzdyy_of_x_y(x,y :double; coe :Tcoe) :double;
begin
  if (x>=coe.xmin)and(x<=coe.xmax)and(y>=coe.ymin)and(y<=coe.ymax) then begin
      ddzdyy_of_x_y:=2*coe.q2;
  end else begin
      ddzdyy_of_x_y:=0;
  end;
end;

procedure Assign_XYZ_of_ExpPoint(i,ic :integer; var x,y,z :double);
begin
      { Assign x,y,z }
      x:=V1[i];
      y:=V2[i];
      case ic of
      3: z:=V3[i];
      4: z:=V4[i];
      5: z:=V5[i];
      6: z:=V6[i];
      7: z:=V7[i];
      8: z:=V8[i];
      end;
      { Shift due to amplifier saturation }
{      case ic of
      3: x:=x-AmplSat.dI[3]*dexp(-abs(y)/AmplSat.Gscale_Of_Sigmoid);
      4: x:=x-AmplSat.dI[4]*dexp(-abs(y)/AmplSat.Gscale_Of_Sigmoid);
      5: x:=x-AmplSat.dI[5]*dexp(-abs(y)/AmplSat.Gscale_Of_Sigmoid);
      6: x:=x-AmplSat.dI[6]*dexp(-abs(y)/AmplSat.Gscale_Of_Sigmoid);
      7: x:=x-AmplSat.dI[7]*dexp(-abs(y)/AmplSat.Gscale_Of_Sigmoid);
      end;}
end;

function ReadCalibrationData :boolean;
var
    i                         :integer;
    bbb                       :text;
    FName                     :string;
begin
  ReadCalibrationData:=false;
  FName:='I_G_VT_dV_subV_PulseV_PulseDur.dat';
  if FileExists(FName) then begin
     assign(bbb,FName);   reset(bbb);
  end;
  if (FileExists(FName)=false)or(eof(bbb)) then begin
     Form7.Memo1.Lines.Add('Warning! Calibration has not been done yet!');
     Form7.Visible:=true;
     Application.ProcessMessages;
     exit;
  end;
  i:=0;
  while not eof(bbb) do begin
    i:=i+1;
    readln(bbb,V1[i],V2[i],V3[i],V4[i],V5[i],V6[i],V7[i],V8[i]);
  end;
  close(bbb);
  Np:=i;  // number of experimental points, i.e. number of pulses
  ReadCalibrationData:=true;
end;

procedure DetermineBoundaries(ic :integer; var coe :Tcoe);
var i                         :integer;
    x,y,z,z_mean              :double;
begin
  coe.xmin:=0; coe.xmax:=0; coe.ymin:=0; coe.ymax:=0;
  for i:=1 to Np do begin
      { Assign x,y,z }
      Assign_XYZ_of_ExpPoint(i,ic, x,y,z);
      { xmin,xmax,ymin,ymax }
      coe.xmin:=min(x,coe.xmin);
      coe.xmax:=max(x,coe.xmax);
      coe.ymin:=min(y,coe.ymin);
      coe.ymax:=max(y,coe.ymax);
      coe.xc:=coe.xc*(i-1)/i+x/i;
      coe.yc:=coe.yc*(i-1)/i+y/i;
      coe.zmean:=coe.zmean*(i-1)/i+z/i;
      coe.z2mean:=coe.z2mean*(i-1)/i+z*z/i;
  end;
end;

procedure Approximate_FC_for_raw_cylinder(ic :integer; var coe :Tcoe);
{ To obtain the approximation "z=k1*xy+b1*x+k2*y+b2"
  one must solve the linear system of equations "A*X=B":
        (x2y2 x2y xy2 xy) (k1)         (xyz)
  Sum_i (x2y  x2  xy  x ) (b1) = Sum_i (xz )
        (xy2  xy  y2  y ) (k2)         (yz )
        (xy   x   y   1 ) (b2)         (z  )
}
var
    i,j,k,nM                  :integer;
    x,y,z                     :double;
    AA                        :matr;
    XX, BB                    :vect;
begin
  { Compose matrices }
  for j:=1 to 4 do for k:=1 to 4 do  AA[j,k]:=0;
  for j:=1 to 4 do BB[j]:=0;
  for i:=1 to Np do begin
      { Assign x,y,z }
      Assign_XYZ_of_ExpPoint(i,ic, x,y,z);
      { Elements }
      AA[1,1]:=AA[1,1]+x*x*y*y/Np;
      AA[1,2]:=AA[1,2]+x*x*y  /Np;
      AA[1,3]:=AA[1,3]+x  *y*y/Np;
      AA[1,4]:=AA[1,4]+x  *y  /Np;
      AA[2,1]:=AA[1,2];
      AA[2,2]:=AA[2,2]+x*x    /Np;
      AA[2,3]:=AA[1,4];
      AA[2,4]:=AA[2,4]+x      /Np;
      AA[3,1]:=AA[1,3];
      AA[3,2]:=AA[1,4];
      AA[3,3]:=AA[3,3]+    y*y/Np;
      AA[3,4]:=AA[3,4]+      y/Np;
      AA[4,1]:=AA[1,4];
      AA[4,2]:=AA[2,4];
      AA[4,3]:=AA[3,4];
      AA[4,4]:=1;
      BB[1]:=BB[1] + x*y*z/Np;
      BB[2]:=BB[2] + x  *z/Np;
      BB[3]:=BB[3] +   y*z/Np;
      BB[4]:=BB[4] +     z/Np;
  end;
  { Solve equation }
  nM:=4;
  LinearSistem(nM, AA, BB, XX);
  coe.q1:=0;
  coe.q2:=0;
  coe.k1:=XX[1];
  coe.b1:=XX[2];
  coe.k2:=XX[3];
  coe.b2:=XX[4];
  DetermineBoundaries(ic,coe);
end;

procedure Approximate_FC_for_raw_linear(ic :integer; var coe :Tcoe);
{ To obtain the approximation "z=b1*x+k2*y+b2"
  one must solve the linear system of equations "A*X=B":
  Sum_i (x2  xy  x ) (b1) = Sum_i (xz )
        (xy  y2  y ) (k2)         (yz )
        (x   y   1 ) (b2)         (z  )
}
var
    i,j,k,nM                  :integer;
    bbb                       :text;
    FName                     :string;
    x,y,z                     :double;
    AA                        :matr;
    XX, BB                    :vect;
begin
  { Compose matrices }
  for j:=1 to 3 do for k:=1 to 3 do  AA[j,k]:=0;
  for j:=1 to 3 do BB[j]:=0;
  for i:=1 to Np do begin
      { Assign x,y,z }
      Assign_XYZ_of_ExpPoint(i,ic, x,y,z);
      { Elements }
      AA[1,1]:=AA[1,1]+x*x    /Np;
      AA[1,2]:=AA[1,2]+x  *y  /Np;
      AA[1,3]:=AA[1,3]+x      /Np;
      AA[2,1]:=AA[1,2];
      AA[2,2]:=AA[2,2]+    y*y/Np;
      AA[2,3]:=AA[2,3]+      y/Np;
      AA[3,1]:=AA[1,3];
      AA[3,2]:=AA[2,3];
      AA[3,3]:=1;
      BB[1]:=BB[1] + x  *z/Np;
      BB[2]:=BB[2] +   y*z/Np;
      BB[3]:=BB[3] +     z/Np;
  end;
  { Solve equation }
  nM:=3;
  LinearSistem(nM, AA, BB, XX);
  coe.q1:=0;
  coe.q2:=0;
  coe.k1:=0;
  coe.b1:=XX[1];
  coe.k2:=XX[2];
  coe.b2:=XX[3];
end;

procedure Approximate_FC_for_raw_quadratic(ic :integer; var coe :Tcoe);
{ To obtain the approximation "z=q1*xx +q2*yy +k1*xy +b1*x +k2*y +b2"
  one must solve the linear system of equations "A*X=B":
        (x4   x2y2 x3y  x3  x2y x2) (q1)         (x2z)
        (x2y2 y4   xy3  xy2 y3  y2) (q2)         (y2z)
  Sum_i (x3y  xy3  x2y2 x2y xy2 xy) (k1) = Sum_i (xyz )
        (x3   xy2  x2y  x2  xy  x)  (b1)         (xz)
        (x2y  y3   xy2  xy  y2  y)  (k2)         (yz )
        (x2   y2   xy   x   y   1)  (b2)         (z  )
}
var
    i,j,k,nM                  :integer;
    x,y,z                     :double;
    AA                        :matr;
    XX, BB                    :vect;
begin
  { Compose matrices }
  for j:=1 to 6 do for k:=1 to 6 do  AA[j,k]:=0;
  for j:=1 to 6 do BB[j]:=0;
  for i:=1 to Np do begin
      { Assign x,y,z }
      Assign_XYZ_of_ExpPoint(i,ic, x,y,z);
      { Elements }
      AA[1,1]:=AA[1,1]+x*x*x*x/Np;
      AA[1,2]:=AA[1,2]+x*x*y*y/Np;
      AA[1,3]:=AA[1,3]+x*x*x*y/Np;
      AA[1,4]:=AA[1,4]+x*x*x  /Np;
      AA[1,5]:=AA[1,5]+x*x*y  /Np;
      AA[1,6]:=AA[1,6]+x*x    /Np;
      AA[2,1]:=AA[1,2];
      AA[2,2]:=AA[2,2]+y*y*y*y/Np;
      AA[2,3]:=AA[2,3]+x*y*y*y/Np;
      AA[2,4]:=AA[2,4]+x*y*y  /Np;
      AA[2,5]:=AA[2,5]+y*y*y  /Np;
      AA[2,6]:=AA[2,6]+y*y    /Np;
      AA[3,1]:=AA[1,3];
      AA[3,2]:=AA[2,3];
      AA[3,3]:=AA[3,3]+x*x*y*y/Np;
      AA[3,4]:=AA[3,4]+x*x*y  /Np;
      AA[3,5]:=AA[3,5]+x  *y*y/Np;
      AA[3,6]:=AA[3,6]+x  *y  /Np;
      AA[4,1]:=AA[1,4];
      AA[4,2]:=AA[2,4];
      AA[4,3]:=AA[3,4];
      AA[4,4]:=AA[4,4]+x*x    /Np;
      AA[4,5]:=AA[3,6];
      AA[4,6]:=AA[4,6]+x      /Np;
      AA[5,1]:=AA[1,5];
      AA[5,2]:=AA[2,5];
      AA[5,3]:=AA[3,5];
      AA[5,4]:=AA[3,6];
      AA[5,5]:=AA[5,5]+    y*y/Np;
      AA[5,6]:=AA[5,6]+      y/Np;
      AA[6,1]:=AA[1,6];
      AA[6,2]:=AA[2,6];
      AA[6,3]:=AA[3,6];
      AA[6,4]:=AA[4,6];
      AA[6,5]:=AA[5,6];
      AA[6,6]:=1;
      BB[1]:=BB[1] + x*x*z/Np;
      BB[2]:=BB[2] + y*y*z/Np;
      BB[3]:=BB[3] + x*y*z/Np;
      BB[4]:=BB[4] + x  *z/Np;
      BB[5]:=BB[5] +   y*z/Np;
      BB[6]:=BB[6] +     z/Np;
  end;
  { Solve equation }
  nM:=6;
  LinearSistem(nM, AA, BB, XX);
  coe.q1:=XX[1];
  coe.q2:=XX[2];
  coe.k1:=XX[3];
  coe.b1:=XX[4];
  coe.k2:=XX[5];
  coe.b2:=XX[6];
  DetermineBoundaries(ic,coe);
end;

{***********************************************************}
function Approximate_FC_for_raw(ic :integer; var coe :Tcoe):boolean;
{ To obtain the approximation "z=k1*xy+b1*x+k2*y+b2"
  one must solve the linear system of equations "A*X=B":
        (x2y2 x2y xy2 xy) (k1)         (xyz)
  Sum_i (x2y  x2  xy  x ) (b1) = Sum_i (xz )
        (xy2  xy  y2  y ) (k2)         (yz )
        (xy   x   y   1 ) (b2)         (z  )
}
begin
  Approximate_FC_for_raw:=false;
  { Read data }
  if ReadCalibrationData=false then exit;
  { Saturation of amplifier - error of injected current }
  AmplSat.dI[3]:=0;  AmplSat.dI[5]:=0;  AmplSat.dI[7]:=0;
  AmplSat.dI[4]:=Form1.DDSpinEdit38.Value;
  AmplSat.dI[6]:=Form1.DDSpinEdit38.Value;
  AmplSat.Gscale_Of_Sigmoid:=Form1.DDSpinEdit39.Value;
  { Approximate }
  case Form1.ComboBox4.ItemIndex of
  { Cylindrical approximation }
  0: Approximate_FC_for_raw_cylinder(ic, coe);
  { Linear approximation }
  1: Approximate_FC_for_raw_linear(ic, coe);
  { Quadratic approximation }
  2: Approximate_FC_for_raw_quadratic(ic, coe);
  { Nearest point from calibration set }
  3: ;
  end;
  Approximate_FC_for_raw:=true;
end;
{***********************************************************}

procedure NearestInCalibrationSet(z1,z2,z3 :double; var x,y :double);
var
    i,ic1,ic2,ic3                   :integer;
    xp,yp,zp1,zp2,zp3, minR,R,R2       :double;
begin
  { Which column to read from file: }
  { 3-VT, 4-PulseV-VT, 5-subV, 6-PulseV, 7-NegPulseDuration }
  ic1:=trunc(Form1.DDSpinEdit9.Value);
  ic2:=trunc(Form1.DDSpinEdit10.Value);
  ic3:=trunc(Form1.DDSpinEdit30.Value);
  minR:=1e8;
  for i:=1 to Np do begin
      { Assign x,y,z }
      Assign_XYZ_of_ExpPoint(i,ic1, xp,yp,zp1);
      Assign_XYZ_of_ExpPoint(i,ic2, xp,yp,zp2);
      Assign_XYZ_of_ExpPoint(i,ic3, xp,yp,zp3);
      R2:=sqr(zp1-z1)+sqr(zp2-z2);
      if Form1.CheckBox21.Checked then R2:=R2+sqr(zp3-z3); // Uze 3 variables
      R:=sqrt(R2);
      if R<minR then begin
         minR:=R;
         x:=xp;
         y:=yp;
      end;
  end;
end;

{---------------- Drawings ----------------------------------------------------}

procedure DrawApproximation1_FC(ic :integer; coe :Tcoe);
var i :integer;
    x,y,z :double;
begin
  { Draw result }
  Form1.Series16.Clear;
  Form1.Series17.Clear;
  for i:=1 to Np do begin
      { Assign x,y,z }
      Assign_XYZ_of_ExpPoint(i,ic, x,y,z);
      Form1.Series16.AddXY(z,z_of_x_y(x,y,coe));
      Form1.Series17.AddXY(z,z);
      Application.ProcessMessages;
  end;
end;

procedure DrawApproximation2_FC(ic :integer; coe :Tcoe);
var i :integer;
    x,y,z :double;
begin
  { Draw result }
  Form1.PointSeries12.Clear;
  Form1.LineSeries2.Clear;
  for i:=1 to Np do begin
      { Assign x,y,z }
      Assign_XYZ_of_ExpPoint(i,ic, x,y,z);
      Form1.PointSeries12.AddXY(z,z_of_x_y(x,y,coe));
      Form1.LineSeries2.AddXY(z,z);
      Application.ProcessMessages;
  end;
end;

var
    z1_view,z2_view,x_view,y_view                :arr_X_Y;
    ni,nj                                        :integer;

procedure DrawLattices(I_min,I_max,G_min,G_max :double);
var i,j,ic1,ic2,ic3,iz_left,iz_right,col :integer;
    x,y,z :double;
    color :TColor;
begin
  { Read calibratrion file and approximate }
  if Approximate_FC=false then exit;
  { Calculate fields }
  ic1:=trunc(Form1.DDSpinEdit9.Value);
  ic2:=trunc(Form1.DDSpinEdit10.Value);
  ic3:=trunc(Form1.DDSpinEdit30.Value);
  iz_left :=Form1.FrameLattice1.ComboBox1.ItemIndex+1;
  iz_right:=Form1.FrameLattice2.ComboBox1.ItemIndex+1;
  if iz_right=3 then ic2:=ic3;
  ni:=100; nj:=80;
  for i:=0 to ni do begin
      for j:=0 to nj do begin
          x_view[i,j]:=I_min+(I_max-I_min)/ni*i;
          y_view[i,j]:=G_min+(G_max-G_min)/nj*j;
          z1_view[i,j]:=z_of_x_y(x_view[i,j],y_view[i,j],AFC.coe[iz_left]);
          z2_view[i,j]:=z_of_x_y(x_view[i,j],y_view[i,j],AFC.coe[iz_right]);
      end;
  end;
  { Draw }
  Form1.FrameLattice1.Draw_Lattice(z1_view,x_view,y_view, ni,nj,0);
  Form1.FrameLattice2.Draw_Lattice(z2_view,x_view,y_view, ni,nj,0);
  { Experimental points }
  for i:=1 to Np do begin
      { Assign x,y,z }
      Assign_XYZ_of_ExpPoint(i,ic1, x,y,z);
      col:=Form1.FrameLattice1.Color_u(z,Form1.FrameLattice1.dmin,
                                         Form1.FrameLattice1.dmax);
      color:=Form1.FrameLattice1.GreenVioletColor(col);
      Form1.FrameLattice1.Series5.AddXY(x,y,'',color);
      {---}
      Assign_XYZ_of_ExpPoint(i,ic2, x,y,z);
      col:=Form1.FrameLattice1.Color_u(z,Form1.FrameLattice2.dmin,
                                         Form1.FrameLattice2.dmax);
      color:=Form1.FrameLattice2.GreenVioletColor(col);
      Form1.FrameLattice2.Series5.AddXY(x,y,'',color);
  end;
  Application.ProcessMessages;
end;

procedure WriteLatticesAndCalibrationPoints;
var i,j,ic1,ic2               :integer;
    x,y,z1_,z2_               :double;
    bbb                       :text;
    FName                     :string;
begin
  { Lattices }
  FName:='Lattices_x_y_z1_z2.dat';
  assign(bbb,FName); rewrite(bbb);
  writeln(bbb,'ZONE T="Lattices_x_y_z1_z2.dat"');
  writeln(bbb,'I= ',nj+1:3,' ,J= ',ni+1:3,', K=1,F=POINT');
  for i:=0 to ni do begin
      for j:=0 to nj do begin
          write  (bbb, x_view[i,j]:10:3,' ', y_view[i,j]:10:3,' ');
          writeln(bbb,z1_view[i,j]:10:3,' ',z2_view[i,j]:10:3);
      end;
  end;
  close(bbb);
  { Calibration points }
  FName:='CalibrationPoints_x_y_z1_z2.dat';
  assign(bbb,FName); rewrite(bbb);
  ic1:=trunc(Form1.DDSpinEdit9.Value);
  ic2:=trunc(Form1.DDSpinEdit10.Value);
  for i:=1 to Np do begin
      { Assign x,y,z }
      Assign_XYZ_of_ExpPoint(i,ic1, x,y,z1_);
      Assign_XYZ_of_ExpPoint(i,ic2, x,y,z2_);
      writeln(bbb, x:10:3,' ', y:10:3,' ',z1_:10:3,' ',z2_:10:3);
  end;
  close(bbb);
end;

end.
