unit EstimateFC;

interface
uses   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
       StdCtrls, DDSpinEdit,
       Init,MathMy,AnalyzeFC,ApproximateFC,Unit1,Unit5,ExtracelStim,Init_DC;

procedure DrawEstimatedPoints;
procedure ReadEstimatedPoints;
procedure Estimate_FC;

implementation

type
  TparsOfEstimation = record
     eps,fi_min :double;
     max_iter :integer;
  end;
var  pars :TparsOfEstimation;
     Cl,U_Cl :double;

procedure EstimatePoint_FC(z1,z2: double; var x,y :double);
{ Newton's method for F(X)-Z =0:
  0 = F(X_old)-Z + dF/dX(X_old) * (X-X_old)
  It is linear system of eqs.:
   AA*dXX = BB, where AA=dF/dX(X_old), dXX=X-X_old, BB=-F(X_old)+Z
}
var
    AA                                         :matr_2x2;
    ZZ,XX,dXX,BB                               :vect_2;
    iter,i                                     :integer;
begin
  { Rename }
  ZZ[1]:=z1; ZZ[2]:=z2;
  { Initial conditions }
  XX[1]:=x; XX[2]:=y;
  { Start iterations }
  iter:=0;
  repeat iter:=iter+1;
    { Jacobian, AA=dF/dX }
    for i:=1 to 2 do begin
        AA[i,1]:=dzdx_of_x_y(XX[1],XX[2],AFC.coe[i]);
        AA[i,2]:=dzdy_of_x_y(XX[1],XX[2],AFC.coe[i]);
    end;
    { Right-hand part }
    for i:=1 to 2 do begin
        BB[i]:=ZZ[i] - z_of_x_y(XX[1],XX[2],AFC.coe[i]);
    end;
    { Linear equation }
    LinearSistem_2x2(AA,BB,dXX);
    { Final Newton's increments }
    for i:=1 to 2 do begin
        XX[i]:=XX[i]+0.5*dXX[i];
    end;
    { Boundaries }
//    XX[1]:=min(XX[1],AFC.coe[1].xmax);    XX[1]:=max(XX[1],AFC.coe[1].xmin);
//    XX[2]:=min(XX[2],AFC.coe[1].ymax);    XX[2]:=max(XX[2],AFC.coe[1].ymin);
  until (abs(BB[1])+abs(BB[2])<pars.eps) or (iter>=pars.max_iter);
  { Result }
  x:=XX[1];
  y:=XX[2];
  if iter>=pars.max_iter then begin x:=0; y:=0; end;
end;

procedure EstimatePoint_FC_3(z1,z2,z3: double; var x,y :double);
{ Minimization of the functional
  F(x,y) = (F1-z1)^2 + (F2-z2)^2 + (F3-z3)^2, where local minimum is defined as
  Fx=dF/dx=0, Fy=dF/dy=0:
  Fx=(F1-z1)*dF1/dx + (F2-z2)*dF2/dx + (F3-z3)*dF3/dx
  Fy=(F1-z1)*dF1/dy + (F2-z2)*dF2/dy + (F3-z3)*dF3/dy, which consist F'=(Fx,Fy).
  Newton's method for F'(X)=0:
  0 = F'(X_old) + dF'/dX(X_old) * (X-X_old)
  It is linear system of eqs.:
  AA*dXX = BB, where AA=dF'/dX(X_old), dXX=X-X_old, BB=-F'(X_old)
}
var
    AA                                         :matr_2x2;
    XX,dXX,BB                                  :vect_2;
    iter,i,k,k_off                             :integer;
    ZZ,F_z,
    dFdx,dFdy,ddFdxx,ddFdxy,ddFdyy
                                               :array[1..3] of double;
    w_                                         :double;
begin
  { Rename }
  ZZ[1]:=z1; ZZ[2]:=z2; ZZ[3]:=z3;
  { Initial conditions }
  XX[1]:=x; XX[2]:=y;
  { Start iterations }
  iter:=0;
  repeat iter:=iter+1;
    for k:=1 to 3 do begin
        w_:=1/sqrt(AFC.coe[k].z2mean-sqr(AFC.coe[k].zmean));
        F_z[k]:=(z_of_x_y(XX[1],XX[2],AFC.coe[k])-ZZ[k])*w_;
        dFdx[k]  :=  dzdx_of_x_y(XX[1],XX[2],AFC.coe[k])*w_;
        dFdy[k]  :=  dzdy_of_x_y(XX[1],XX[2],AFC.coe[k])*w_;
        ddFdxx[k]:=ddzdxx_of_x_y(XX[1],XX[2],AFC.coe[k])*w_;
        ddFdxy[k]:=ddzdxy_of_x_y(XX[1],XX[2],AFC.coe[k])*w_;
        ddFdyy[k]:=ddzdyy_of_x_y(XX[1],XX[2],AFC.coe[k])*w_;
    end;
    { Switch off one of functions 2 or 3 }
    k_off:=3;
    if abs(dFdx[1]*dFdy[2]-dFdy[1]*dFdx[2])/sqrt(sqr(dFdx[2])+sqr(dFdy[2]))
                                           /sqrt(sqr(dFdx[1])+sqr(dFdy[1]))<pars.fi_min
       then if sqr(dFdx[1])+sqr(dFdy[1])<sqr(dFdx[2])+sqr(dFdy[2]) then k_off:=1
                                                                   else k_off:=2;
{        min(sqr(dFdx[2])+sqr(dFdy[2]),sqr(dFdx[3])+sqr(dFdy[3])) then k_off:=1
    else if sqr(dFdx[2])+sqr(dFdy[2])<
        min(sqr(dFdx[1])+sqr(dFdy[1]),sqr(dFdx[3])+sqr(dFdy[3])) then k_off:=2
    else                                                              k_off:=3;
       sqr(dFdx[1])+sqr(dFdy[1])+sqr(dFdx[3])+sqr(dFdy[3]) then k_off:=1 else
    if (sqrt(sqr(dFdx[2])+sqr(dFdy[2]))>sqrt(sqr(dFdx[3])+sqr(dFdy[3])))
    if (sqrt(sqr(dFdx[2])+sqr(dFdy[2]))>sqrt(sqr(dFdx[3])+sqr(dFdy[3])))
  then k_off:=3
                                                    else k_off:=2;}
    F_z[k_off]:=0; dFdx[k_off]:=0; dFdy[k_off]:=0;
    ddFdxx[k_off]:=0; ddFdxy[k_off]:=0; ddFdyy[k_off]:=0;
    { Jacobian, AA=dF/dX }
    AA[1,1]:=sqr(dFdx[1]) + F_z[1]*ddFdxx[1]
           + sqr(dFdx[2]) + F_z[2]*ddFdxx[2]
           + sqr(dFdx[3]) + F_z[3]*ddFdxx[3];
    AA[1,2]:=dFdx[1]*dFdy[1] + F_z[1]*ddFdxy[1]
           + dFdx[2]*dFdy[2] + F_z[2]*ddFdxy[2]
           + dFdx[3]*dFdy[3] + F_z[3]*ddFdxy[3];
    AA[2,1]:=AA[1,2];
    AA[2,2]:=sqr(dFdy[1]) + F_z[1]*ddFdyy[1]
           + sqr(dFdy[2]) + F_z[2]*ddFdyy[2]
           + sqr(dFdy[3]) + F_z[3]*ddFdyy[3];
    { Right-hand part }
    BB[1]:=-F_z[1]*dFdx[1] - F_z[2]*dFdx[2] - F_z[3]*dFdx[3];
    BB[2]:=-F_z[1]*dFdy[1] - F_z[2]*dFdy[2] - F_z[3]*dFdy[3];
    { Linear equation }
    LinearSistem_2x2(AA,BB,dXX);
    { Final Newton's increments }
    for i:=1 to 2 do begin
        XX[i]:=XX[i]+0.5*dXX[i];
    end;
    { Boundaries }
//    XX[1]:=min(XX[1],AFC.coe[1].xmax);    XX[1]:=max(XX[1],AFC.coe[1].xmin);
//    XX[2]:=min(XX[2],AFC.coe[1].ymax);    XX[2]:=max(XX[2],AFC.coe[1].ymin);
  until (abs(BB[1])+abs(BB[2])<pars.eps) or (iter>=pars.max_iter);
  { Result }
  x:=XX[1];
  y:=XX[2];
  if iter>=pars.max_iter then begin x:=0; y:=0; end;
end;

function WhichVariable(ic,iPulse :integer) :double;
{ Which variables were approximated to estimate I,G: }
{ 3-VT, 4-PulseV-VT, 5-subV, 6-PulseV, 7-NegPulseDuration, 8-dVdt }
var z :double;
begin
  case ic of
    3: z:=AFC.VT[iPulse];
    4: z:=AFC.VAP[iPulse];
    5: z:=AFC.subV[iPulse];
    6: z:=AFC.PulseV[iPulse];
    7: z:=AFC.NegPulseDuration[iPulse];
    8: z:=AFC.dVdt[iPulse];
  end;
  WhichVariable:=z;
end;

function EQForVGABA(iPulse :integer; GI_:double):double;
{ Estimated the dynamics of VGABA by the eq. derived on p.86 of the GreenBook
  (the one with experimental protocols from Umea started in Nov. 2011.) }
var Cl8,dt_,tau1_,k_,tau2_,RT_zF,Cl_out :double;
begin
  { Parameters }
  RT_zF:=25.35{26.64};
  Cl_out:=146{130}; {mM}
  Cl8:=Cl_out*exp(-80/RT_zF); {mM}
  dt_:=AFC.PulseTime[2]-AFC.PulseTime[1];
  tau1_  :=Form5.DDSpinEdit7.Value;  {ms}       // 1000
  tau2_  :=Form5.DDSpinEdit9.Value;  {ms}       // 0
  k_     :=Form5.DDSpinEdit8.Value;  {mM/s/nS}  // 0.12
  { Initial condition }
  if iPulse=1 then begin Cl:=Cl8; U_Cl:=0; end;
  {*****************************************************}
  if tau2_=0 then begin
     Cl:=Cl + dt_*(k_*GI_ + Cl8 - Cl)/tau1_;
  end else begin
     New_m3(0,1/tau1_,1/tau2_,dt_,Cl8+k_*GI_, Cl,U_Cl);
  end;
  {*****************************************************}
  if Cl>0 then
  EQForVGABA:=RT_zF*ln(Cl/Cl_out);
end;

procedure Estimate_GE_GI;
var
    iPulse                                        :integer;
    GE_,GI_,VE_,VI_,VL_,f_,q_,VE_VI_,Veff_,Mg_    :double;
begin
  VE_  :=Form5.DDSpinEdit2.Value; {mV}
  VI_  :=Form5.DDSpinEdit3.Value; {mV}
  VL_  :=Form5.DDSpinEdit11.Value; {mV}
  Mg_  :=Form5.DDSpinEdit12.Value; {mM}
  Veff_:=Form5.DDSpinEdit13.Value; {mV}
  for iPulse:=1 to AFC.nPulse do begin
      { NMDA nonlinearity, p.76 of Green Book }
      if Form5.CheckBox4.Checked {NMDA} then begin
         f_:=1/(1+Mg_/3.57*dexp(-0.062*Veff_));
         q_:=f_+(Veff_-VE_)*Mg_/3.57*dexp(-0.062*Veff_)*0.062*sqr(f_);
         VE_VI_:=(VE_-Veff_)*f_ + (Veff_-Vus)*q_;
         { Consideration simila to Lyle's one of NaP-current effect }
{         f_:=1+0.017*Mg_*Vus;
         q_:=(Veff_-VE_)*0.017*Mg_;     if Mg_=0 then q_:=1;
         VE_VI_:=(VE_-Veff_)*f_;   }
      end else begin
         q_:=1;
         VE_VI_:=VE_-VI_;
      end;
      {******** Estimate conductances *******************************}
      GE_:=(AFC.Estim_I[iPulse]-AFC.Estim_G[iPulse]*(VI_-Vus))/VE_VI_;
      GI_:= AFC.Estim_G[iPulse]-GE_*q_;
      {**************************************************************}
      { Estimate VGABA and correct GE_ }
      if Form5.CheckBox2.Checked {V_syn} then begin
         AFC.VGABA_estimated[iPulse]:=EqForVGABA(iPulse,GI_);
         GE_:=(AFC.Estim_I[iPulse]
              -AFC.Estim_G[iPulse]*(AFC.VGABA_estimated[iPulse]-Vus))
                                  /VE_VI_;//(-AFC.VGABA_estimated[iPulse]); 28.02.2013
         GI_:= AFC.Estim_G[iPulse]-GE_*q_;
      end;
      { Reconstruct_V_from_GE_GI }
      AFC.V_from_GE_GI[iPulse]:=(PP.G_L*VL_+GE_*VE_+GI_*VI_)/(PP.G_L+GE_+GI_);
      AFC.Estim_GE[iPulse]:=GE_;
      AFC.Estim_GI[iPulse]:=GI_;
  end;
end;

procedure DrawEstimatedPoints;
var
    iPulse,iStim              :integer;
    tP,GE_,GI_                :double;
    ok                        :boolean;
begin
  Form5.Chart4.Visible:=Form5.CheckBox3.Checked; //V_from_GE_GI
  { if show lines }
  ok:=Form5.CheckBox1.Checked;
  Form5.Series1.LinePen.Visible:=ok;
  Form5.Series2.LinePen.Visible:=ok;
  Form5.Series3.LinePen.Visible:=ok;
  Form5.Series4.LinePen.Visible:=ok;
  Form5.Series5.LinePen.Visible:=ok;
  Form5.Series6.LinePen.Visible:=ok;
  Form5.Series7.LinePen.Visible:=ok;
  Form5.Series8.LinePen.Visible:=ok;
  Form5.Series9.LinePen.Visible:=ok;
  Form5.Series10.LinePen.Visible:=ok;
  Form5.Series11.LinePen.Visible:=ok;
  Form5.Series12.LinePen.Visible:=ok;
  Form5.Series13.LinePen.Visible:=ok;
  Form5.Series14.LinePen.Visible:=ok;
  Form5.Series15.LinePen.Visible:=ok;
  Form5.Series16.LinePen.Visible:=ok;
  Form5.Series17.LinePen.Visible:=ok;
  { Draw }
  Form5.Series1.Clear;
  Form5.Series2.Clear;
  Form5.Series3.Clear;
  Form5.Series4.Clear;
  Form5.Series9.Clear;
  Form5.Series10.Clear;
  Form5.Series13.Clear;
  Form5.Series14.Clear;
  Form5.Series15.Clear;
  Form5.Series16.Clear;
  Form5.Series17.Clear;
  for iPulse:=1 to AFC.nPulse do begin
      tP:=AFC.PulseTime[iPulse];
      { Align to stimuli }
      if Form1.CheckBox14.Checked then NumberOfStimul(tP,iStim);
      Form5.Series1.AddXY(tP,AFC.Estim_I[iPulse]);
      Form5.Series2.AddXY(tP,AFC.Estim_G[iPulse]);
      Form5.Series3.AddXY(tP,AFC.TrueStim_I[iPulse]);
      Form5.Series4.AddXY(tP,AFC.TrueStim_G[iPulse]);
      Form5.Series9.AddXY(tP,AFC.z2[iPulse]);
      Form5.Series10.AddXY(tP,AFC.z1[iPulse]);
      GE_:=AFC.Estim_GE[iPulse];
      GI_:=AFC.Estim_GI[iPulse];
      {***************************************************************}
      Form5.Series13.AddXY(tP,GE_);
      Form5.Series14.AddXY(tP,GI_);
      { Reversal potential V_syn }
      if (Form5.CheckBox2.Checked)and(AFC.Estim_G[iPulse]>0.1{nS}) then begin
          Form5.Series15.AddXY(tP,Vus+AFC.Estim_I[iPulse]/AFC.Estim_G[iPulse]);
          Form5.Series16.AddXY(tP,AFC.VGABA_estimated[iPulse]);
      end;
      { Reconstruct_V_from_GE_GI }
      if Form5.CheckBox3.Checked then begin   //V_from_GE_GI
         Form5.Series17.AddXY(tP,AFC.V_from_GE_GI[iPulse]);
      end;
  end;
  Application.ProcessMessages;
end;

procedure ReadEstimatedPoints;
var FName :string;
    iPulse,iStim :integer;
    aaa :text;
begin
  FName:='TrueAndEstimated_I_G.dat';
  assign(aaa,FName); reset(aaa);
  iPulse:=0;
  repeat iPulse:=iPulse+1;
    read  (aaa,AFC.TrueStim_I[iPulse],AFC.TrueStim_G[iPulse]);
    read  (aaa,AFC.Estim_I[iPulse],AFC.Estim_G[iPulse]);
    read  (aaa,AFC.PulseTime[iPulse]);
    if not(eoln(aaa)) then
    read  (aaa,AFC.z1[iPulse],AFC.z2[iPulse]);
    read  (aaa,AFC.Estim_GE[iPulse],AFC.Estim_GI[iPulse]);
    read  (aaa,AFC.VGABA_estimated[iPulse],AFC.V_from_GE_GI[iPulse]);
    readln(aaa);
    if Form1.CheckBox14.Checked then NumberOfStimul(AFC.PulseTime[iPulse],iStim);
  until eof(aaa);
  AFC.nPulse:=iPulse;
  close(aaa);
end;

procedure WriteEstimatedPoints;
var FName         :string;
    iPulse,iStim  :integer;
    tP            :double;
    bbb           :text;
begin
  FName:='TrueAndEstimated_I_G.dat';
  assign(bbb,FName); rewrite(bbb);
  for iPulse:=1 to AFC.nPulse do begin
      tP:=AFC.PulseTime[iPulse];
      if Form1.CheckBox14.Checked then NumberOfStimul(tP,iStim);  // Align to stimuli
      {               A-1                            B-2                       }
      write  (bbb,AFC.TrueStim_I[iPulse]:8:3,' ',AFC.TrueStim_G[iPulse]:8:3,' ');
      {               C-3                         D-4                          }
      write  (bbb,AFC.Estim_I[iPulse]:8:3,' ',AFC.Estim_G[iPulse]:8:3,' ');
      {           E-5            F-6                    G-7                    }
      write  (bbb,tP:8:3,' ',AFC.z1[iPulse]:8:3,' ',AFC.z2[iPulse]:8:3,' ');
      {               H-8                          I-9                         }
      write  (bbb,AFC.Estim_GE[iPulse]:8:3,' ',AFC.Estim_GI[iPulse]:8:3,' ');
      {               J-10                                K-11                 }
      writeln(bbb,AFC.VGABA_estimated[iPulse]:8:3,' ',AFC.V_from_GE_GI[iPulse]:8:3);
  end;
  close(bbb);
end;


{*********************************************************************}
{*********************************************************************}
procedure VersionWithAmplifierDelayTau(iPulse :integer);
{ Correction of AFC.Estim_G[iPulse] and AFC.Estim_I[iPulse] after estimation,
  assuming certain delay of amplifier. tau1=C1*Rs, i.e. tau1/C/Rs=C1/C}
var
    Rs_,G_,x,y,wG,wI1,wI2,C1C   :double;
begin
  if Offline.G1=0   then Rs_:=0  else Rs_:=1/Offline.G1;   {GOhm}
  C1C:=Offline.C1_C;
  x:=Rs_*AFC.Estim_G[iPulse];
  y:=Rs_*PP.G_L;
  wG:=(C1C*y/(1+x)+1)/(1+x-C1C*x/(1+x));
  {************************************************************}
  AFC.Estim_G[iPulse]:=AFC.Estim_G[iPulse]*wG;
  {************************************************************}
  wI1:=             1/(1+x-C1C*x/(1+x));
  wI2:=wI1-1;
  if Form1.ComboBox5.ItemIndex=5 then wI2:=0;  { without Meander }
  {*********************************************************************}
  AFC.Estim_I[iPulse]:=AFC.Estim_I[iPulse]*wI1 + AFC.Meander[iPulse]*wI2;
  {*********************************************************************}
end;

procedure ArtifactConductanceAndCapacitanceCompensation(iPulse :integer);
{ Correction of AFC.Estim_G[iPulse] and AFC.Estim_I[iPulse] after estimation. }
var
    Rs_,G_,x,y,z,wG,wI1,wI2,b   :double;
begin
  AFC.Estim_G[iPulse]:=AFC.Estim_G[iPulse] - Offline.dG_offset;
  AFC.Estim_I[iPulse]:=AFC.Estim_I[iPulse] - Offline.dI_offset;
  if Form1.ComboBox5.ItemIndex in [5,6] then begin // C&G compensation after Estimation
     VersionWithAmplifierDelayTau(iPulse);
     exit;
  end;
  if not(Form1.ComboBox5.ItemIndex in [3,4]) then exit;
  if Offline.G1=0   then Rs_:=0  else Rs_:=1/Offline.G1;   {GOhm}
  if Offline.C1_C=0 then b:=1000 else b:=1/Offline.C1_C;
  x:=Rs_*AFC.Estim_G[iPulse];
  y:=Rs_*PP.G_L;
  wG:=((b+1)*x+(2+b)*x*y+(b+3)*y+x*y*y+2*y*y+b+1)/(b+1+(b-1)*x-x*x-2*x*y-x*x*y);
  {*****************************************************************}
  AFC.Estim_G[iPulse]:=AFC.Estim_G[iPulse]*wG;// - Offline.dG_offset;
  {*****************************************************************}
  z:=Rs_*AFC.Estim_G[iPulse];
  wI1:=(x+x*z+y*z+y*y*x+b*x+b*y+2*y*x+y*x*z+b*y*x+b+1+2*y+y*y+z)/(b+1+y+b*x);
  wI2:=(x+x*z+y*y*x-z*b+2*y*x+y*x*z-z*b*x)                      /(b+1+y+b*x);
  if Form1.ComboBox5.ItemIndex=4 then wI2:=0;  { without Meander }
  {*********************************************************************}
  AFC.Estim_I[iPulse]:=AFC.Estim_I[iPulse]*wI1 + AFC.Meander[iPulse]*wI2;
//                                             - Offline.dI_offset;
  {*********************************************************************}
end;

function dV_AmplSatur(ic :integer; G_ :double) :double;
{ Shift due to amplifier saturation }
begin
  dV_AmplSatur:=0;
  if (G_>0)and(AmplSat.Gscale_Of_Sigmoid>0) then
  case ic of
    4: dV_AmplSatur:=-AmplSat.dI[4]/G_*(1-dexp(-abs(G_)/AmplSat.Gscale_Of_Sigmoid));
    6: dV_AmplSatur:=-AmplSat.dI[6]/{G_}PP.G_L*(1-dexp(-abs(G_)/AmplSat.Gscale_Of_Sigmoid));
  end;
end;

{*********************************************************************}
{*********************************************************************}

procedure Estimate_FC;
var
    iPulse,max_iter,iz1,iz2,iz3           :integer;
    x,y,G_,w_                             :double;
begin
  { Parameters }
  pars.eps     :=Form1.DDSpinEdit31.Value;//0.01{mV or ms};
  pars.fi_min  :=Form1.DDSpinEdit32.Value;
  pars.max_iter:=trunc(Form1.DDSpinEdit33.Value);
  iz1:=trunc(Form1.DDSpinEdit9 .Value);
  iz2:=trunc(Form1.DDSpinEdit10.Value);
  iz3:=trunc(Form1.DDSpinEdit30.Value);
  { Estimation }
  for iPulse:=1 to AFC.nPulse do begin
      AFC.z1[iPulse]:=WhichVariable(iz1,iPulse);
      AFC.z2[iPulse]:=WhichVariable(iz2,iPulse);
      AFC.z3[iPulse]:=WhichVariable(iz3,iPulse);
      { Amplifier Saturation voltage shift }
      if iPulse>1 then begin
         G_:=PP.G_L+AFC.Estim_G[iPulse-1];
         AFC.z1[iPulse]:=AFC.z1[iPulse]+dV_AmplSatur(iz1,G_);
         AFC.z2[iPulse]:=AFC.z2[iPulse]+dV_AmplSatur(iz2,G_);
         AFC.z3[iPulse]:=AFC.z3[iPulse]+dV_AmplSatur(iz3,G_);
      end;
      { Filter z1,z2 }
      if (Offline.w_Filter_z1_z2<>0) and (iPulse>1) then begin
         w_:=Offline.w_Filter_z1_z2;
         AFC.z1[iPulse]:=AFC.z1[iPulse]*(1-w_)+AFC.z1[iPulse-1]*w_;
         AFC.z2[iPulse]:=AFC.z2[iPulse]*(1-w_)+AFC.z2[iPulse-1]*w_;
         AFC.z3[iPulse]:=AFC.z3[iPulse]*(1-w_)+AFC.z3[iPulse-1]*w_;
      end;
      { Initial conditions }
      if iPulse>1 then begin
         x:=(AFC.Estim_I[iPulse-1]+AFC.coe[1].xc)/2;
         y:=(AFC.Estim_G[iPulse-1]+AFC.coe[1].yc)/2;
      end else begin x:=0; y:=0; end;
      { Reverse problem }
      if Form1.ComboBox4.ItemIndex=3 then begin // Nearest point in calibration set
         NearestInCalibrationSet(AFC.z1[iPulse],AFC.z2[iPulse],AFC.z3[iPulse],x,y);
      end else begin
         if Form1.CheckBox21.Checked then begin // Uze 3 variables
            EstimatePoint_FC_3(AFC.z1[iPulse],AFC.z2[iPulse],AFC.z3[iPulse], x,y);
         end else begin                         // Uze 2 variables
            EstimatePoint_FC(AFC.z1[iPulse],AFC.z2[iPulse], x,y);
         end;
      end;
      AFC.Estim_I[iPulse]:=x;
      AFC.Estim_G[iPulse]:=y;
  end;
  { Compensation }
  for iPulse:=1 to AFC.nPulse do begin
      ArtifactConductanceAndCapacitanceCompensation(iPulse);
  end;
  {*************}
  Estimate_GE_GI;
  {*************}
  { Write }
  WriteEstimatedPoints;
  { Draw }
  DrawEstimatedPoints;
end;

end.
